from typing import Union, List

import numpy as np
from numba import vectorize, float64

from pythermalcomfort.models import cooling_effect
from pythermalcomfort.shared_functions import valid_range
from pythermalcomfort.utilities import (
    units_converter,
    check_standard_compliance_array,
)


def pmv_ppd(
    tdb: Union[float, int, np.ndarray, List[float], List[int]],
    tr: Union[float, int, np.ndarray, List[float], List[int]],
    vr: Union[float, int, np.ndarray, List[float], List[int]],
    rh: Union[float, int, np.ndarray, List[float], List[int]],
    met: Union[float, int, np.ndarray, List[float], List[int]],
    clo: Union[float, int, np.ndarray, List[float], List[int]],
    wme: Union[float, int, np.ndarray, List[float], List[int]] = 0,
    standard: str = "ISO",
    units: str = "SI",
    limit_inputs: bool = True,
    airspeed_control: bool = True,
):
    """Returns Predicted Mean Vote (`PMV`_) and Predicted Percentage of
    Dissatisfied ( `PPD`_) calculated in accordance to main thermal comfort
    Standards. The PMV is an index that predicts the mean value of the thermal
    sensation votes (self-reported perceptions) of a large group of people on a
    sensation scale expressed from –3 to +3 corresponding to the categories:
    cold, cool, slightly cool, neutral, slightly warm, warm, and hot. [1]_

    While the PMV equation is the same for both the ISO and ASHRAE standards, in the
    ASHRAE 55 PMV equation, the SET is used to calculate the cooling effect first,
    this is then subtracted from both the air and mean radiant temperatures, and the
    differences are used as input to the PMV model, while the airspeed is set to 0.1m/s.
    Please read more in the Note below.

    Parameters
    ----------
    tdb : float, int, or array-like
        dry bulb air temperature, default in [°C] in [°F] if `units` = 'IP'
    tr : float, int, or array-like
        mean radiant temperature, default in [°C] in [°F] if `units` = 'IP'
    vr : float, int, or array-like
        relative air speed, default in [m/s] in [fps] if `units` = 'IP'

        Note: vr is the relative air speed caused by body movement and not the air
        speed measured by the air speed sensor. The relative air speed is the sum of the
        average air speed measured by the sensor plus the activity-generated air speed
        (Vag). Where Vag is the activity-generated air speed caused by motion of
        individual body parts. vr can be calculated using the function
        :py:meth:`pythermalcomfort.utilities.v_relative`.
    rh : float, int, or array-like
        relative humidity, [%]
    met : float, int, or array-like
        metabolic rate, [met]
    clo : float, int, or array-like
        clothing insulation, [clo]

        Note: The activity as well as the air speed modify the insulation characteristics
        of the clothing and the adjacent air layer. Consequently, the ISO 7730 states that
        the clothing insulation shall be corrected [2]_. The ASHRAE 55 Standard corrects
        for the effect of the body movement for met equal or higher than 1.2 met using
        the equation clo = Icl × (0.6 + 0.4/met) The dynamic clothing insulation, clo,
        can be calculated using the function
        :py:meth:`pythermalcomfort.utilities.clo_dynamic`.
    wme : float, int, or array-like
        external work, [met] default 0
    standard : str, optional
        select comfort standard used for calculation.
        Supported values are 'ASHRAE' and 'ISO'. Defaults to 'ISO'.

        - If "ISO", then the ISO Equation is used
        - If "ASHRAE", then the ASHRAE Equation is used

        Note: While the PMV equation is the same for both the ISO and ASHRAE standards,
        the ASHRAE Standard Use of the PMV model is limited to air speeds below 0.10
        m/s (20 fpm).
        When air speeds exceed 0.10 m/s (20 fpm), the comfort zone boundaries are
        adjusted based on the SET model.
        This change was indroduced by the `Addendum C to Standard 55-2020`_
    units : str, optional
        select the SI (International System of Units) or the IP (Imperial Units) system.
        Supported values are 'SI' and 'IP'. Defaults to 'SI'.
    limit_inputs : boolean default True
        By default, if the inputs are outsude the standard applicability limits the
        function returns nan. If False returns pmv and ppd values even if input values are
        outside the applicability limits of the model.

        The ASHRAE 55 2020 limits are 10 < tdb [°C] < 40, 10 < tr [°C] < 40,
        0 < vr [m/s] < 2, 1 < met [met] < 4, and 0 < clo [clo] < 1.5.
        The ISO 7730 2005 limits are 10 < tdb [°C] < 30, 10 < tr [°C] < 40,
        0 < vr [m/s] < 1, 0.8 < met [met] < 4, 0 < clo [clo] < 2, and -2 < PMV < 2.
    airspeed_control : boolean default True
        This only applies if standard = "ASHRAE". By default it is assumed that the
        occupant has control over the airspeed. In this case the ASHRAE 55 Standard does
        not impose any airspeed limits. On the other hand, if the occupant has no control
        over the airspeed the ASHRAE 55 imposes an upper limit for v which varies as a
        function of the operative temperature, for more information please consult the
        Standard.

    Returns
    -------
    pmv : float, int, or array-like
        Predicted Mean Vote
    ppd : float, int, or array-like
        Predicted Percentage of Dissatisfied occupants, [%]

    Notes
    -----
    You can use this function to calculate the `PMV`_ and `PPD`_ in accordance with
    either the ASHRAE 55 2020 Standard [1]_ or the ISO 7730 Standard [2]_.

    .. _PMV: https://en.wikipedia.org/wiki/Thermal_comfort#PMV/PPD_method
    .. _PPD: https://en.wikipedia.org/wiki/Thermal_comfort#PMV/PPD_method
    .. _Addendum C to Standard 55-2020: https://www.ashrae.org/file%20library/technical%20resources/standards%20and%20guidelines/standards%20addenda/55_2020_c_20210430.pdf

    Examples
    --------
    .. code-block:: python

        >>> from pythermalcomfort.models import pmv_ppd
        >>> from pythermalcomfort.utilities import v_relative, clo_dynamic
        >>> tdb = 25
        >>> tr = 25
        >>> rh = 50
        >>> v = 0.1
        >>> met = 1.4
        >>> clo = 0.5
        >>> # calculate relative air speed
        >>> v_r = v_relative(v=v, met=met)
        >>> # calculate dynamic clothing
        >>> clo_d = clo_dynamic(clo=clo, met=met)
        >>> results = pmv_ppd(tdb=tdb, tr=tr, vr=v_r, rh=rh, met=met, clo=clo_d)
        >>> print(results)
        {'pmv': 0.06, 'ppd': 5.1}
        >>> print(results["pmv"])
        -0.06
        >>> # you can also pass an array-like of inputs
        >>> results = pmv_ppd(tdb=[22, 25], tr=tr, vr=v_r, rh=rh, met=met, clo=clo_d)
        >>> print(results)
        {'pmv': array([-0.47,  0.06]), 'ppd': array([9.6, 5.1])}

    Raises
    ------
    StopIteration
        Raised if the number of iterations exceeds the threshold
    ValueError
        The 'standard' function input parameter can only be 'ISO' or 'ASHRAE'
    """

    tdb = np.asarray(tdb)
    tr = np.asarray(tr)
    vr = np.asarray(vr)
    met = np.asarray(met)
    clo = np.asarray(clo)
    wme = np.asarray(wme)

    if units.lower() == "ip":
        tdb, tr, vr = units_converter(tdb=tdb, tr=tr, v=vr)

    standard = standard.lower()
    if standard not in ["iso", "ashrae"]:
        raise ValueError(
            "PMV calculations can only be performed in compliance with ISO or ASHRAE "
            "Standards"
        )

    (
        tdb_valid,
        tr_valid,
        v_valid,
        met_valid,
        clo_valid,
    ) = check_standard_compliance_array(
        standard,
        tdb=tdb,
        tr=tr,
        v=vr,
        met=met,
        clo=clo,
        airspeed_control=airspeed_control,
    )

    # if v_r is higher than 0.1 follow methodology ASHRAE Appendix H, H3
    ce = 0.0
    if standard == "ashrae":
        ce = np.where(
            vr > 0.1,
            np.vectorize(cooling_effect, cache=True)(tdb, tr, vr, rh, met, clo, wme),
            0.0,
        )

    tdb = tdb - ce
    tr = tr - ce
    vr = np.where(ce > 0, 0.1, vr)

    pmv_array = _pmv_ppd_optimized(tdb, tr, vr, rh, met, clo, wme)

    ppd_array = 100.0 - 95.0 * np.exp(
        -0.03353 * pmv_array**4.0 - 0.2179 * pmv_array**2.0
    )

    # Checks that inputs are within the bounds accepted by the model if not return nan
    if limit_inputs:
        pmv_valid = valid_range(pmv_array, (-2, 2))  # this is the ISO limit
        if standard == "ashrae":
            pmv_valid = valid_range(pmv_array, (-100, 100))

        all_valid = ~(
            np.isnan(tdb_valid)
            | np.isnan(tr_valid)
            | np.isnan(v_valid)
            | np.isnan(met_valid)
            | np.isnan(clo_valid)
            | np.isnan(pmv_valid)
        )
        pmv_array = np.where(all_valid, pmv_array, np.nan)
        ppd_array = np.where(all_valid, ppd_array, np.nan)

    return {
        "pmv": np.around(pmv_array, 2),
        "ppd": np.around(ppd_array, 1),
    }


@vectorize(
    [
        float64(
            float64,
            float64,
            float64,
            float64,
            float64,
            float64,
            float64,
        )
    ],
)
def _pmv_ppd_optimized(tdb, tr, vr, rh, met, clo, wme):
    pa = rh * 10 * np.exp(16.6536 - 4030.183 / (tdb + 235))

    icl = 0.155 * clo  # thermal insulation of the clothing in M2K/W
    m = met * 58.15  # metabolic rate in W/M2
    w = wme * 58.15  # external work in W/M2
    mw = m - w  # internal heat production in the human body
    # calculation of the clothing area factor
    if icl <= 0.078:
        f_cl = 1 + (1.29 * icl)  # ratio of surface clothed body over nude body
    else:
        f_cl = 1.05 + (0.645 * icl)

    # heat transfer coefficient by forced convection
    hcf = 12.1 * np.sqrt(vr)
    hc = hcf  # initialize variable
    taa = tdb + 273
    tra = tr + 273
    t_cla = taa + (35.5 - tdb) / (3.5 * icl + 0.1)

    p1 = icl * f_cl
    p2 = p1 * 3.96
    p3 = p1 * 100
    p4 = p1 * taa
    p5 = (308.7 - 0.028 * mw) + (p2 * (tra / 100.0) ** 4)
    xn = t_cla / 100
    xf = t_cla / 50
    eps = 0.00015

    n = 0
    while np.abs(xn - xf) > eps:
        xf = (xf + xn) / 2
        hcn = 2.38 * np.abs(100.0 * xf - taa) ** 0.25
        if hcf > hcn:
            hc = hcf
        else:
            hc = hcn
        xn = (p5 + p4 * hc - p2 * xf**4) / (100 + p3 * hc)
        n += 1
        if n > 150:
            raise StopIteration("Max iterations exceeded")

    tcl = 100 * xn - 273

    # heat loss diff. through skin
    hl1 = 3.05 * 0.001 * (5733 - (6.99 * mw) - pa)
    # heat loss by sweating
    if mw > 58.15:
        hl2 = 0.42 * (mw - 58.15)
    else:
        hl2 = 0
    # latent respiration heat loss
    hl3 = 1.7 * 0.00001 * m * (5867 - pa)
    # dry respiration heat loss
    hl4 = 0.0014 * m * (34 - tdb)
    # heat loss by radiation
    hl5 = 3.96 * f_cl * (xn**4 - (tra / 100.0) ** 4)
    # heat loss by convection
    hl6 = f_cl * hc * (tcl - tdb)

    ts = 0.303 * np.exp(-0.036 * m) + 0.028
    _pmv = ts * (mw - hl1 - hl2 - hl3 - hl4 - hl5 - hl6)

    return _pmv



# -*- coding: utf-8 -*-
"""This code defines a dataclass for default input and a dictionary called ALL_OUT_PARAMS that contains
information about various output parameters related to human body properties,
heat exchange, and environmental conditions.

It also includes a function called show_outparam_docs() that generates a formatted string with the documentation
of the output parameters.

The show_outparam_docs() function uses text wrapping to create a readable documentation string
for both regular output parameters and extra output parameters.

It sorts the parameters alphabetically by key and formats each line with the parameter's name, meaning, and unit.
The resulting documentation string can be displayed or printed for user reference.
"""
from typing import List

import textwrap
from dataclasses import dataclass
from typing import ClassVar

import numpy as np


@dataclass(frozen=True)
class Default:
    # Body information
    height: float = 1.72  # [m]
    weight: float = 74.43  # [kg]
    age: int = 20  # [-]
    body_fat: float = 15  # [%]
    cardiac_index: float = 2.59  # [L/min/m2]
    blood_flow_rate: int = 290  # [L/h]
    physical_activity_ratio: float = 1.25  # [-]
    metabolic_rate: float = 1.0  # [met]
    sex: str = "male"
    posture: str = "standing"
    bmr_equation: str = "harris-benedict"
    bsa_equation: str = "dubois"
    local_bsa: ClassVar[List[float]] = np.array(  # body surface area [m2]
        [
            0.110,
            0.029,
            0.175,
            0.161,
            0.221,
            0.096,
            0.063,
            0.050,
            0.096,
            0.063,
            0.050,
            0.209,
            0.112,
            0.056,
            0.209,
            0.112,
            0.056,
        ]
    )

    # Environment information
    core_temperature: float = 37  # [°C]
    skin_temperature: float = 34  # [°C]
    other_body_temperature: float = 36  # [°C]
    dry_bulb_air_temperature: float = 28.8  # [°C]
    mean_radiant_temperature: float = 28.8  # [°C]
    relative_humidity: float = 50  # [%]
    air_speed: float = 0.1  # [m/s]
    # Clothing information
    clothing_insulation: float = 0  # [clo]
    clothing_vapor_permeation_efficiency: float = 0.45  # [-]
    lewis_rate = 16.5  # [K/kPa]


ALL_OUT_PARAMS = {
    "age": {"ex_output": True, "meaning": "age", "suffix": None, "unit": "years"},
    "bf_ava_foot": {
        "ex_output": True,
        "meaning": "AVA blood flow rate of one foot",
        "suffix": None,
        "unit": "L/h",
    },
    "bf_ava_hand": {
        "ex_output": True,
        "meaning": "AVA blood flow rate of one hand",
        "suffix": None,
        "unit": "L/h",
    },
    "bf_core": {
        "ex_output": True,
        "meaning": "core blood flow rate (each body part)",
        "suffix": "Body name",
        "unit": "L/h",
    },
    "bf_fat": {
        "ex_output": True,
        "meaning": "fat blood flow rate (each body part)",
        "suffix": "Body name",
        "unit": "L/h",
    },
    "bf_muscle": {
        "ex_output": True,
        "meaning": "muscle blood flow rate (each body part)",
        "suffix": "Body name",
        "unit": "L/h",
    },
    "bf_skin": {
        "ex_output": True,
        "meaning": "skin blood flow rate (each body part)",
        "suffix": "Body name",
        "unit": "L/h",
    },
    "bsa": {
        "ex_output": True,
        "meaning": "body surface area (each body part)",
        "suffix": "Body name",
        "unit": "m2",
    },
    "cardiac_output": {
        "ex_output": False,
        "meaning": "cardiac output (the sum of the whole blood flow)",
        "suffix": None,
        "unit": "L/h",
    },
    "cycle_time": {
        "ex_output": False,
        "meaning": "the counts of executing one cycle calculation",
        "suffix": None,
        "unit": "-",
    },
    "e_max": {
        "ex_output": True,
        "meaning": "maximum evaporative heat loss from the skin (each body part)",
        "suffix": "Body name",
        "unit": "W",
    },
    "e_skin": {
        "ex_output": True,
        "meaning": "evaporative heat loss from the skin (each body part)",
        "suffix": "Body name",
        "unit": "W",
    },
    "e_sweat": {
        "ex_output": True,
        "meaning": (
            "evaporative heat loss from the skin by only sweating (each body part)"
        ),
        "suffix": "Body name",
        "unit": "W",
    },
    "fat": {"ex_output": True, "meaning": "body fat rate", "suffix": None, "unit": "%"},
    "height": {
        "ex_output": True,
        "meaning": "body height",
        "suffix": None,
        "unit": "m",
    },
    "clo": {
        "ex_output": True,
        "meaning": "clothing insulation (each body part)",
        "suffix": "Body name",
        "unit": "clo",
    },
    "q_skin2env_latent": {
        "ex_output": True,
        "meaning": "latent heat loss from the skin (each body part)",
        "suffix": "Body name",
        "unit": "W",
    },
    "q_bmr_core": {
        "ex_output": True,
        "meaning": "core thermogenesis by basal metabolism (each body part)",
        "suffix": "Body name",
        "unit": "W",
    },
    "q_bmr_fat": {
        "ex_output": True,
        "meaning": "fat thermogenesis by basal metabolism (each body part)",
        "suffix": "Body name",
        "unit": "W",
    },
    "q_bmr_muscle": {
        "ex_output": True,
        "meaning": "muscle thermogenesis by basal metabolism (each body part)",
        "suffix": "Body name",
        "unit": "W",
    },
    "q_bmr_skin": {
        "ex_output": True,
        "meaning": "skin thermogenesis by basal metabolism (each body part)",
        "suffix": "Body name",
        "unit": "W",
    },
    "q_thermogenesis_total": {
        "ex_output": False,
        "meaning": "total thermogenesis of the whole body",
        "suffix": None,
        "unit": "W",
    },
    "q_nst": {
        "ex_output": True,
        "meaning": "core thermogenesis by non-shivering (each body part)",
        "suffix": "Body name",
        "unit": "W",
    },
    "simulation_time": {
        "ex_output": False,
        "meaning": "simulation times",
        "suffix": None,
        "unit": "sec",
    },
    "q_shiv": {
        "ex_output": True,
        "meaning": "core or muscle thermogenesis by shivering (each body part)",
        "suffix": "Body name",
        "unit": "W",
    },
    "q_work": {
        "ex_output": True,
        "meaning": "core or muscle thermogenesis by work (each body part)",
        "suffix": "Body name",
        "unit": "W",
    },
    "name": {
        "ex_output": True,
        "meaning": "name of the model",
        "suffix": None,
        "unit": "-",
    },
    "par": {
        "ex_output": True,
        "meaning": "physical activity ratio",
        "suffix": None,
        "unit": "-",
    },
    "q_thermogenesis_core": {
        "ex_output": True,
        "meaning": "core total thermogenesis (each body part)",
        "suffix": "Body name",
        "unit": "W",
    },
    "q_thermogenesis_fat": {
        "ex_output": True,
        "meaning": "fat total thermogenesis (each body part)",
        "suffix": "Body name",
        "unit": "W",
    },
    "q_thermogenesis_muscle": {
        "ex_output": True,
        "meaning": "muscle total thermogenesis (each body part)",
        "suffix": "Body name",
        "unit": "W",
    },
    "q_thermogenesis_skin": {
        "ex_output": True,
        "meaning": "skin total thermogenesis (each body part)",
        "suffix": "Body name",
        "unit": "W",
    },
    "q_res": {
        "ex_output": False,
        "meaning": "heat loss by respiration",
        "suffix": None,
        "unit": "W",
    },
    "q_res_latent": {
        "ex_output": True,
        "meaning": "latent heat loss by respiration (each body part)",
        "suffix": "Body name",
        "unit": "W",
    },
    "q_res_sensible": {
        "ex_output": True,
        "meaning": "sensible heat loss by respiration (each body part)",
        "suffix": "Body name",
        "unit": "W",
    },
    "rh": {
        "ex_output": True,
        "meaning": "relative humidity (each body part)",
        "suffix": "Body name",
        "unit": "%",
    },
    "r_et": {
        "ex_output": True,
        "meaning": "total clothing evaporative heat resistance (each body part)",
        "suffix": "Body name",
        "unit": "(m2*kPa)/W",
    },
    "r_t": {
        "ex_output": True,
        "meaning": "total clothing heat resistance (each body part)",
        "suffix": "Body name",
        "unit": "(m2*K)/W",
    },
    "q_skin2env_sensible": {
        "ex_output": True,
        "meaning": "sensible heat loss from the skin (each body part)",
        "suffix": "Body name",
        "unit": "W",
    },
    "t_skin_set": {
        "ex_output": True,
        "meaning": "skin set point temperature (each body part)",
        "suffix": "Body name",
        "unit": "°C",
    },
    "t_core_set": {
        "ex_output": True,
        "meaning": "core set point temperature (each body part)",
        "suffix": "Body name",
        "unit": "°C",
    },
    "sex": {
        "ex_output": True,
        "meaning": "sex",
        "suffix": None,
        "unit": "-",
    },
    "q_skin2env": {
        "ex_output": False,
        "meaning": "total heat loss from the skin (each body part)",
        "suffix": "Body name",
        "unit": "W",
    },
    "tdb": {
        "ex_output": True,
        "meaning": "dry bulb air temperature (each body part)",
        "suffix": "Body name",
        "unit": "°C",
    },
    "t_artery": {
        "ex_output": True,
        "meaning": "arterial temperature (each body part)",
        "suffix": "Body name",
        "unit": "°C",
    },
    "t_cb": {
        "ex_output": True,
        "meaning": "central blood temperature",
        "suffix": None,
        "unit": "°C",
    },
    "t_core": {
        "ex_output": False,
        "meaning": "core temperature (each body part)",
        "suffix": "Body name",
        "unit": "°C",
    },
    "t_fat": {
        "ex_output": True,
        "meaning": "fat temperature (each body part)",
        "suffix": "Body name",
        "unit": "°C",
    },
    "t_muscle": {
        "ex_output": True,
        "meaning": "muscle temperature (each body part)",
        "suffix": "Body name",
        "unit": "°C",
    },
    "to": {
        "ex_output": True,
        "meaning": "operative temperature (each body part)",
        "suffix": "Body name",
        "unit": "°C",
    },
    "tr": {
        "ex_output": True,
        "meaning": "mean radiant temperature (each body part)",
        "suffix": "Body name",
        "unit": "°C",
    },
    "t_skin": {
        "ex_output": False,
        "meaning": "skin temperature (each body part)",
        "suffix": "Body name",
        "unit": "°C",
    },
    "t_skin_mean": {
        "ex_output": False,
        "meaning": "mean skin temperature",
        "suffix": None,
        "unit": "°C",
    },
    "t_superficial_vein": {
        "ex_output": True,
        "meaning": "superficial vein temperature (each body part)",
        "suffix": "Body name",
        "unit": "°C",
    },
    "t_vein": {
        "ex_output": True,
        "meaning": "vein temperature (each body part)",
        "suffix": "Body name",
        "unit": "°C",
    },
    "v": {
        "ex_output": True,
        "meaning": "air velocity (each body part)",
        "suffix": "Body name",
        "unit": "m/s",
    },
    "weight": {
        "ex_output": True,
        "meaning": "body weight",
        "suffix": None,
        "unit": "kg",
    },
    "w": {
        "ex_output": False,
        "meaning": "skin wettedness (each body part)",
        "suffix": "Body name",
        "unit": "-",
    },
    "w_mean": {
        "ex_output": False,
        "meaning": "mean skin wettedness",
        "suffix": None,
        "unit": "-",
    },
    "weight_loss_by_evap_and_res": {
        "ex_output": False,
        "meaning": "weight loss by the evaporation and respiration of the whole body",
        "suffix": None,
        "unit": "g/sec",
    },
    "dt": {
        "ex_output": False,
        "meaning": "time step",
        "suffix": None,
        "unit": "sec",
    },
    "pythermalcomfort_version": {
        "ex_output": False,
        "meaning": "version of pythermalcomfort",
        "suffix": None,
        "unit": "-",
    },
}


def show_out_param_docs():
    """Show the documentation of the output parameters.

    Returns
    -------
    docstring : str
        Text of the documentation of the output parameters
    """

    outparams = textwrap.dedent(
        """
        Output parameters
        -------
        """
    )

    exoutparams = textwrap.dedent(
        """
        Extra output parameters
        -------
        """
    )

    sortkeys = list(ALL_OUT_PARAMS.keys())
    sortkeys.sort()
    for key in sortkeys:
        value = ALL_OUT_PARAMS[key]

        line = "{}: {} [{}]".format(key.ljust(8), value["meaning"], value["unit"])

        if value["ex_output"]:
            exoutparams += line + "\n"
        else:
            outparams += line + "\n"

    docs = outparams + "\n" + exoutparams
    docs = textwrap.indent(docs.strip(), "    ")

    return docs


if __name__ == "__main__":
    print(show_out_param_docs())



import numpy as np

from pythermalcomfort.models import discomfort_index


def test_discomfort_index():
    np.testing.assert_equal(
        discomfort_index([21, 23.5, 29, 32, 35, 40], 50),
        {
            "di": [19.2, 21.0, 25.0, 27.2, 29.4, 33.0],
            "discomfort_condition": [
                "No discomfort",
                "Less than 50% feels discomfort",
                "More than 50% feels discomfort",
                "Most of the population feels discomfort",
                "Everyone feels severe stress",
                "State of medical emergency",
            ],
        },
    )
    np.testing.assert_equal(
        discomfort_index([35, 35], [10, 90]),
        {
            "di": [24.9, 33.9],
            "discomfort_condition": [
                "More than 50% feels discomfort",
                "State of medical emergency",
            ],
        },
    )



from dataclasses import dataclass
from typing import Union, List

import numpy as np

from pythermalcomfort.shared_functions import valid_range

c_to_k = 273.15
cp_vapour = 1805.0
cp_water = 4186
cp_air = 1004
h_fg = 2501000
r_air = 287.055
g = 9.81  # m/s2


def p_sat_torr(tdb: Union[float, int, np.ndarray, List[float], List[int]]):
    """Estimates the saturation vapor pressure in [torr]

    Parameters
    ----------
    tdb : float, int, or array-like
        dry bulb air temperature, [C]

    Returns
    -------
    p_sat : float
        saturation vapor pressure [torr]
    """
    return np.exp(18.6686 - 4030.183 / (tdb + 235.0))


def t_o(
    tdb: Union[float, int, np.ndarray, List[float], List[int]],
    tr: Union[float, int, np.ndarray, List[float], List[int]],
    v: Union[float, int, np.ndarray, List[float], List[int]],
    standard: str = "ISO",
):
    """Calculates operative temperature in accordance with ISO 7726:1998 [5]_

    Parameters
    ----------
    tdb: float, int, or array-like
        air temperature, [°C]
    tr: float, int, or array-like
        mean radiant temperature, [°C]
    v: float, int, or array-like
        air speed, [m/s]
    standard: str (default="ISO")
        either choose between ISO and ASHRAE


    Returns
    -------
    to: float
        operative temperature, [°C]
    """

    if standard.lower() == "iso":
        return (tdb * np.sqrt(10 * v) + tr) / (1 + np.sqrt(10 * v))
    elif standard.lower() == "ashrae":
        a = np.where(v < 0.6, 0.6, 0.7)
        a = np.where(v < 0.2, 0.5, a)
        return a * tdb + (1 - a) * tr


def enthalpy(
    tdb: Union[float, int, np.ndarray, List[float], List[int]],
    hr: Union[float, int, np.ndarray, List[float], List[int]],
):
    """Calculates air enthalpy

    Parameters
    ----------
    tdb: float, int, or array-like
        air temperature, [°C]
    hr: float, int, or array-like
        humidity ratio, [kg water/kg dry air]

    Returns
    -------
    enthalpy: float, int, or array-like
        enthalpy [J/kg dry air]
    """

    h_dry_air = cp_air * tdb
    h_sat_vap = h_fg + cp_vapour * tdb
    h = h_dry_air + hr * h_sat_vap

    return round(h, 2)


def p_sat(tdb: Union[float, int, np.ndarray, List[float], List[int]]):
    """Calculates vapour pressure of water at different temperatures

    Parameters
    ----------
    tdb: float, int, or array-like
        air temperature, [°C]

    Returns
    -------
    p_sat: float, int, or array-like
        saturation vapor pressure, [Pa]
    """

    ta_k = tdb + c_to_k
    c1 = -5674.5359
    c2 = 6.3925247
    c3 = -0.9677843 * 1e-2
    c4 = 0.62215701 * 1e-6
    c5 = 0.20747825 * 1e-8
    c6 = -0.9484024 * 1e-12
    c7 = 4.1635019
    c8 = -5800.2206
    c9 = 1.3914993
    c10 = -0.048640239
    c11 = 0.41764768 * 1e-4
    c12 = -0.14452093 * 1e-7
    c13 = 6.5459673

    pascals = np.where(
        ta_k < c_to_k,
        np.exp(
            c1 / ta_k
            + c2
            + ta_k * (c3 + ta_k * (c4 + ta_k * (c5 + c6 * ta_k)))
            + c7 * np.log(ta_k)
        ),
        np.exp(
            c8 / ta_k
            + c9
            + ta_k * (c10 + ta_k * (c11 + ta_k * c12))
            + c13 * np.log(ta_k)
        ),
    )

    return np.around(pascals, 1)


@dataclass
class PsychrometricValues:
    p_sat: Union[float, int, np.ndarray, List[float], List[int]]
    p_vap: Union[float, int, np.ndarray, List[float], List[int]]
    hr: Union[float, int, np.ndarray, List[float], List[int]]
    t_wb: Union[float, int, np.ndarray, List[float], List[int]]
    t_dp: Union[float, int, np.ndarray, List[float], List[int]]
    h: Union[float, int, np.ndarray, List[float], List[int]]

    def __getitem__(self, item):
        return getattr(self, item)


def psy_ta_rh(
    tdb: Union[float, int, np.ndarray, List[float], List[int]],
    rh: Union[float, int, np.ndarray, List[float], List[int]],
    p_atm=101325,
) -> PsychrometricValues:
    """Calculates psychrometric values of air based on dry bulb air temperature and
    relative humidity.
    For more accurate results we recommend the use of the Python package
    `psychrolib`_.

    .. _psychrolib: https://pypi.org/project/PsychroLib/

    Parameters
    ----------
    tdb: float, int, or array-like
        air temperature, [°C]
    rh: float, int, or array-like
        relative humidity, [%]
    p_atm: float, int, or array-like
        atmospheric pressure, [Pa]

    Returns
    -------
    p_vap: float, int, or array-like
        partial pressure of water vapor in moist air, [Pa]
    hr: float, int, or array-like
        humidity ratio, [kg water/kg dry air]
    t_wb: float, int, or array-like
        wet bulb temperature, [°C]
    t_dp: float, int, or array-like
        dew point temperature, [°C]
    h: float, int, or array-like
        enthalpy [J/kg dry air]
    """
    tdb = np.array(tdb)
    rh = np.array(rh)

    p_saturation = p_sat(tdb)
    p_vap = rh / 100 * p_saturation
    hr = 0.62198 * p_vap / (p_atm - p_vap)
    tdp = t_dp(tdb, rh)
    twb = t_wb(tdb, rh)
    h = enthalpy(tdb, hr)

    return PsychrometricValues(
        p_sat=p_saturation,
        p_vap=p_vap,
        hr=hr,
        t_wb=twb,
        t_dp=tdp,
        h=h,
    )


def t_wb(
    tdb: Union[float, int, np.ndarray, List[float], List[int]],
    rh: Union[float, int, np.ndarray, List[float], List[int]],
):
    """Calculates the wet-bulb temperature using the Stull equation [6]_

    Parameters
    ----------
    tdb: float, int, or array-like
        air temperature, [°C]
    rh: float, int, or array-like
        relative humidity, [%]

    Returns
    -------
    tdb: float, int, or array-like
        wet-bulb temperature, [°C]
    """
    twb = (
        tdb * np.arctan(0.151977 * (rh + 8.313659) ** 0.5)
        + np.arctan(tdb + rh)
        - np.arctan(rh - 1.676331)
        + 0.00391838 * rh**1.5 * np.arctan(0.023101 * rh)
        - 4.686035
    )

    return np.around(twb, 1)


def t_dp(
    tdb: Union[float, int, np.ndarray, List[float], List[int]],
    rh: Union[float, int, np.ndarray, List[float], List[int]],
):
    """Calculates the dew point temperature.

    Parameters
    ----------
    tdb: float, int, or array-like
        dry bulb air temperature, [°C]
    rh: float, int, or array-like
        relative humidity, [%]

    Returns
    -------
    t_dp: float, int, or array-like
        dew point temperature, [°C]
    """
    tdb = np.array(tdb)
    rh = np.array(rh)

    c = 257.14
    b = 18.678
    d = 234.5

    gamma_m = np.log(rh / 100 * np.exp((b - tdb / d) * (tdb / (c + tdb))))

    return np.round(c * gamma_m / (b - gamma_m), 1)


def t_mrt(
    tg: Union[float, int, np.ndarray, List[float], List[int]],
    tdb: Union[float, int, np.ndarray, List[float], List[int]],
    v: Union[float, int, np.ndarray, List[float], List[int]],
    d: Union[float, int, np.ndarray, List[float], List[int]] = 0.15,
    emissivity: Union[float, int, np.ndarray, List[float], List[int]] = 0.95,
    standard="Mixed Convection",
):
    """Converts globe temperature reading into mean radiant temperature in accordance
    with either the Mixed Convection developed by Teitelbaum E. et al. (2022) or the ISO
    7726:1998 Standard [5]_.

    Parameters
    ----------
    tg : float, int, or array-like
        globe temperature, [°C]
    tdb : float, int, or array-like
        air temperature, [°C]
    v : float, int, or array-like
        air speed, [m/s]
    d : float, int, or array-like
        diameter of the globe, [m] default 0.15 m
    emissivity : float, int, or array-like
        emissivity of the globe temperature sensor, default 0.95
    standard : str, optional
        Supported values are 'Mixed Convection' and 'ISO'. Defaults to 'Mixed Convection'.
        either choose between the Mixed Convection and ISO formulations.
        The Mixed Convection formulation has been proposed by Teitelbaum E. et al. (2022)
        to better determine the free and forced convection coefficient used in the
        calculation of the mean radiant temperature. They also showed that mean radiant
        temperature measured with ping-pong ball-sized globe thermometers is not reliable
        due to a stochastic convective bias [22]_. The Mixed Convection model has only
        been validated for globe sensors with a diameter between 0.04 and 0.15 m.

    Returns
    -------
    tr: float, int, or array-like
        mean radiant temperature, [°C]
    """
    standard = standard.lower()

    tdb = np.array(tdb)
    tg = np.array(tg)
    v = np.array(v)
    d = np.array(d)

    if standard == "mixed convection":
        mu = 0.0000181  # Pa s
        k_air = 0.02662  # W/m-K
        beta = 0.0034  # 1/K
        nu = 0.0000148  # m2/s
        alpha = 0.00002591  # m2/s
        pr = cp_air * mu / k_air  # Prandtl constants

        o = 0.0000000567
        n = 1.27 * d + 0.57

        ra = g * beta * np.absolute(tg - tdb) * d * d * d / nu / alpha
        re = v * d / nu

        nu_natural = 2 + (0.589 * np.power(ra, (1 / 4))) / (
            np.power(1 + np.power(0.469 / pr, 9 / 16), (4 / 9))
        )
        nu_forced = 2 + (
            0.4 * np.power(re, 0.5) + 0.06 * np.power(re, 2 / 3)
        ) * np.power(pr, 0.4)

        tr = (
            np.power(
                np.power(tg + 273.15, 4)
                - (
                    (
                        (
                            (
                                np.power(
                                    (np.power(nu_forced, n) + np.power(nu_natural, n)),
                                    1 / n,
                                )
                            )
                            * k_air
                            / d
                        )
                        * (-tg + tdb)
                    )
                    / emissivity
                    / o
                ),
                0.25,
            )
            - 273.15
        )

        d_valid = valid_range(d, (0.04, 0.15))
        tr = np.where(~np.isnan(d_valid), tr, np.nan)

        return np.around(tr, 1)

    if standard == "iso":

        tg = np.add(tg, c_to_k)
        tdb = np.add(tdb, c_to_k)

        # calculate heat transfer coefficient
        h_n = np.power(1.4 * (np.abs(tg - tdb) / d), 0.25)  # natural convection
        h_f = 6.3 * np.power(v, 0.6) / np.power(d, 0.4)  # forced convection

        # get the biggest between the two coefficients
        h = np.maximum(h_f, h_n)

        tr = (
            np.power(
                np.power(tg, 4) + h * (tg - tdb) / (emissivity * (5.67 * 10**-8)),
                0.25,
            )
            - c_to_k
        )

        return np.around(tr, 1)



from itertools import product

import numpy as np
import pytest

from pythermalcomfort.models import cooling_effect


def test_cooling_effect():

    t_range = np.arange(10, 40, 10)
    rh_range = np.arange(10, 75, 25)
    v_range = np.arange(0.1, 4, 1)
    all_combinations = list(product(t_range, rh_range, v_range))
    results = [
        0,
        8.19,
        10.94,
        12.54,
        0,
        8.05,
        10.77,
        12.35,
        0,
        7.91,
        10.6,
        12.16,
        0,
        5.04,
        6.62,
        7.51,
        0,
        4.84,
        6.37,
        7.24,
        0,
        4.64,
        6.12,
        6.97,
        0,
        3.64,
        4.32,
        4.69,
        0,
        3.55,
        4.25,
        4.61,
        0,
        3.4,
        4.1,
        4.46,
    ]
    for ix, comb in enumerate(all_combinations):
        pytest.approx(
            cooling_effect(
                tdb=comb[0],
                tr=comb[0],
                rh=comb[1],
                vr=comb[2],
                met=1,
                clo=0.5,
            )
            == results[ix],
            0.1,
        )

    assert (cooling_effect(tdb=25, tr=25, vr=0.05, rh=50, met=1, clo=0.6)) == 0
    assert (cooling_effect(tdb=25, tr=25, vr=0.5, rh=50, met=1, clo=0.6)) == 2.17
    assert (cooling_effect(tdb=27, tr=25, vr=0.5, rh=50, met=1, clo=0.6)) == 1.85
    assert (cooling_effect(tdb=29, tr=25, vr=0.5, rh=50, met=1, clo=0.6)) == 1.63
    assert (cooling_effect(tdb=31, tr=25, vr=0.5, rh=50, met=1, clo=0.6)) == 1.42
    assert (cooling_effect(tdb=25, tr=27, vr=0.5, rh=50, met=1, clo=0.6)) == 2.44
    assert (cooling_effect(tdb=25, tr=29, vr=0.5, rh=50, met=1, clo=0.6)) == 2.81
    assert (cooling_effect(tdb=25, tr=25, vr=0.2, rh=50, met=1, clo=0.6)) == 0.67
    assert (cooling_effect(tdb=25, tr=25, vr=0.8, rh=50, met=1, clo=0.6)) == 2.93
    assert (cooling_effect(tdb=25, tr=25, vr=0.0, rh=50, met=1, clo=0.6)) == 0
    assert (cooling_effect(tdb=25, tr=25, vr=0.5, rh=60, met=1, clo=0.6)) == 2.13
    assert (cooling_effect(tdb=25, tr=25, vr=0.5, rh=80, met=1, clo=0.6)) == 2.06
    assert (cooling_effect(tdb=25, tr=25, vr=0.5, rh=20, met=1, clo=0.6)) == 2.29
    assert (cooling_effect(tdb=25, tr=25, vr=0.5, rh=60, met=1.3, clo=0.6)) == 2.84
    assert (cooling_effect(tdb=25, tr=25, vr=0.5, rh=60, met=1.6, clo=0.6)) == 3.5
    assert (cooling_effect(tdb=25, tr=25, vr=0.5, rh=60, met=1, clo=0.3)) == 2.41
    assert (cooling_effect(tdb=25, tr=25, vr=0.5, rh=60, met=1, clo=1)) == 2.05

    # test what happens when the cooling effect cannot be calculated
    assert (cooling_effect(tdb=0, tr=80, vr=5, rh=60, met=3, clo=1)) == 0

    assert (
        cooling_effect(tdb=77, tr=77, vr=1.64, rh=50, met=1, clo=0.6, units="IP")
    ) == 3.95



from pythermalcomfort.models.heat_index import heat_index
from pythermalcomfort.models.pet_steady import pet_steady
from pythermalcomfort.models.wc import wc
from pythermalcomfort.models.humidex import humidex
from pythermalcomfort.models.at import at
from pythermalcomfort.models.solar_gain import solar_gain
from pythermalcomfort.models.cooling_effect import cooling_effect
from pythermalcomfort.models.pmv_ppd import pmv_ppd
from pythermalcomfort.models.pmv import pmv
from pythermalcomfort.models.a_pmv import a_pmv
from pythermalcomfort.models.e_pmv import e_pmv
from pythermalcomfort.models.set_tmp import set_tmp
from pythermalcomfort.models.two_nodes import two_nodes
from pythermalcomfort.models.use_fans_heatwaves import use_fans_heatwaves
from pythermalcomfort.models.adaptive_ashrae import adaptive_ashrae
from pythermalcomfort.models.adaptive_en import adaptive_en
from pythermalcomfort.models.utci import utci
from pythermalcomfort.models.vertical_tmp_grad_ppd import vertical_tmp_grad_ppd
from pythermalcomfort.models.clo_tout import clo_tout
from pythermalcomfort.models.ankle_draft import ankle_draft
from pythermalcomfort.models.phs import phs
from pythermalcomfort.models.wbgt import wbgt
from pythermalcomfort.models.net import net
from pythermalcomfort.models.discomfort_index import discomfort_index
from pythermalcomfort.models.athb import athb
from pythermalcomfort.models.jos3 import JOS3
from pythermalcomfort.models.atcs import ATCS
from pythermalcomfort.models.zhang_comfort import zhang_sensation_comfort



import numpy as np

from pythermalcomfort.models import pmv


def e_pmv(tdb, tr, vr, rh, met, clo, e_coefficient, wme=0, **kwargs):
    """Returns Adjusted Predicted Mean Votes with Expectancy Factor (ePMV).
    This index was developed by Fanger, P. O. et al. (2002). In non-air-
    conditioned buildings in warm climates, occupants may sense the warmth as
    being less severe than the PMV predicts. The main reason is low
    expectations, but a metabolic rate that is estimated too high can also
    contribute to explaining the difference. An extension of the PMV model that
    includes an expectancy factor is introduced for use in non-air-conditioned
    buildings in warm climates [26]_.

    Parameters
    ----------
    tdb : float, int, or array-like
        dry bulb air temperature, default in [°C] in [°F] if `units` = 'IP'
    tr : float, int, or array-like
        mean radiant temperature, default in [°C] in [°F] if `units` = 'IP'
    vr : float, int, or array-like
        relative air speed, default in [m/s] in [fps] if `units` = 'IP'

        Note: vr is the relative air speed caused by body movement and not the air
        speed measured by the air speed sensor. The relative air speed is the sum of the
        average air speed measured by the sensor plus the activity-generated air speed
        (Vag). Where Vag is the activity-generated air speed caused by motion of
        individual body parts. vr can be calculated using the function
        :py:meth:`pythermalcomfort.utilities.v_relative`.
    rh : float, int, or array-like
        relative humidity, [%]
    met : float, int, or array-like
        metabolic rate, [met]
    clo : float, int, or array-like
        clothing insulation, [clo]

        Note: The activity as well as the air speed modify the insulation characteristics
        of the clothing and the adjacent air layer. Consequently, the ISO 7730 states that
        the clothing insulation shall be corrected [2]_. The ASHRAE 55 Standard corrects
        for the effect of the body movement for met equal or higher than 1.2 met using
        the equation clo = Icl × (0.6 + 0.4/met) The dynamic clothing insulation, clo,
        can be calculated using the function
        :py:meth:`pythermalcomfort.utilities.clo_dynamic`.
    e_coefficient : float
        expectacy factor
    wme : float, int, or array-like
        external work, [met] default 0

    Other Parameters
    ----------------
    units : {'SI', 'IP'}
        select the SI (International System of Units) or the IP (Imperial Units) system.
    limit_inputs : boolean default True
        By default, if the inputs are outsude the standard applicability limits the
        function returns nan. If False returns pmv and ppd values even if input values are
        outside the applicability limits of the model.

        The ISO 7730 2005 limits are 10 < tdb [°C] < 30, 10 < tr [°C] < 40,
        0 < vr [m/s] < 1, 0.8 < met [met] < 4, 0 < clo [clo] < 2, and -2 < PMV < 2.

    Returns
    -------
    pmv : float, int, or array-like
        Predicted Mean Vote

    Examples
    --------
    .. code-block:: python

        >>> from pythermalcomfort.models import a_pmv
        >>> from pythermalcomfort.utilities import v_relative, clo_dynamic
        >>> tdb = 28
        >>> tr = 28
        >>> rh = 50
        >>> v = 0.1
        >>> met = 1.4
        >>> clo = 0.5
        >>> # calculate relative air speed
        >>> v_r = v_relative(v=v, met=met)
        >>> # calculate dynamic clothing
        >>> clo_d = clo_dynamic(clo=clo, met=met)
        >>> results = e_pmv(tdb, tr, v_r, rh, met, clo_d, e_coefficient=0.6)
        >>> print(results)
        0.51
    """
    default_kwargs = {"units": "SI", "limit_inputs": True}
    kwargs = {**default_kwargs, **kwargs}

    met = np.array(met)
    _pmv = pmv(tdb, tr, vr, rh, met, clo, wme, "ISO", **kwargs)
    met = np.where(_pmv > 0, met * (1 + _pmv * (-0.067)), met)
    _pmv = pmv(tdb, tr, vr, rh, met, clo, wme, "ISO", **kwargs)

    return np.around(_pmv * e_coefficient, 2)



import pandas as pd
import re


def convert_and_print_local_clo_values_from_csv_to_dict(csv_name):
    """Parameters
    ----------
    csv_name : file path that you want to convert (it should be in the same folder as this function)

    Returns
    -------
    local_clo_dict : a dictionary including local clothing insulation values as well as that for the whole body

    Notes
    -----
    The 17 body segments correspond to the JOS-3 model.
    """
    # Read the Excel file
    df = pd.read_csv(csv_name)

    # Create an empty dictionary
    local_clo_dict = {}

    # Add data from each row to the dictionary
    for index, row in df.iterrows():
        # Get the name of the clothing combination as the key
        key = row["clothing_ensemble"]
        # Create a dictionary as the value
        value = {"whole_body": row["whole_body"], "local_body_part": {}}
        # Add data from each column to the Local body dictionary
        for col in df.columns[2:]:
            # Get the name of the body part
            body_part = col
            # Get the clo value for the local body part and add it to the Local body dictionary
            clo = row[col]
            value["local_body_part"][body_part] = clo
        # Add the data to the dictionary
        local_clo_dict[key] = value

    return local_clo_dict


def add_prompt_to_code(code: str, prompt: str = ">>> ") -> str:
    lines = code.strip().split("\n")
    result = []
    for line in lines:
        if re.match(r"^\s*#", line):  # If it's a comment line
            result.append(line)
        else:
            result.append(prompt + line)
    return "\n".join(result)


sample_code = """
# Build a model and set a body built
# Create an instance of the JOS3 class with optional body parameters such as body height, weight, age, sex, etc.
model = JOS3(
    height=1.7,
    weight=60,
    fat=20,
    age=30,
    sex="male",
    bmr_equation="japanese",
    bsa_equation="fujimoto",
    ex_output="all",
)

# Set environmental conditions such as air temperature, mean radiant temperature using the setter methods.
# Set the first condition
# Environmental parameters can be input as int, float, list, dict, numpy array format.
model.tdb = 28  # Air temperature [°C]
model.tr = 30  # Mean radiant temperature [°C]
model.rh = 40  # Relative humidity [%]
model.v = np.array( # Air velocity [m/s]
    [
        0.2,  # head
        0.4,  # neck
        0.4,  # chest
        0.1,  # back
        0.1,  # pelvis
        0.4,  # left shoulder
        0.4,  # left arm
        0.4,  # left hand
        0.4,  # right shoulder
        0.4,  # right arm
        0.4,  # right hand
        0.1,  # left thigh
        0.1,  # left leg
        0.1,  # left foot
        0.1,  # right thigh
        0.1,  # right leg
        0.1,  # right foot
    ]
)
model.clo = local_clo_typical_ensembles["briefs, socks, undershirt, work jacket, work pants, safety shoes"]["local_body_part"]

# par should be input as int, float.
model.par = 1.2  # Physical activity ratio [-], assuming a sitting position
# posture should be input as int (0, 1, or 2) or str ("standing", "sitting" or "lying").
# (0="standing", 1="sitting" or 2="lying")
model.posture = "sitting"  # Posture [-], assuming a sitting position

# Run JOS-3 model
model.simulate(
    times=30,  # Number of loops of a simulation
    dtime=60,  # Time delta [sec]. The default is 60.
)  # Exposure time = 30 [loops] * 60 [sec] = 30 [min]

# Set the next condition (You only need to change the parameters that you want to change)
model.to = 20  # Change operative temperature
model.v = { # Air velocity [m/s], assuming to use a desk fan
    'head' : 0.2,
    'neck' : 0.4,
    'chest' : 0.4,
    'back': 0.1,
    'pelvis' : 0.1,
    'left_shoulder' : 0.4,
    'left_arm' : 0.4,
    'left_hand' : 0.4,
    'right_shoulder' : 0.4,
    'right_arm' : 0.4,
    'right_hand' : 0.4,
    'left_thigh' : 0.1,
    'left_leg' : 0.1,
    'left_foot' : 0.1,
    'right_thigh' : 0.1,
    'right_leg' : 0.1,
    'right_foot' : 0.1
    }
# Run JOS-3 model
model.simulate(
    times=60,  # Number of loops of a simulation
    dtime=60,  # Time delta [sec]. The default is 60.
)  # Additional exposure time = 60 [loops] * 60 [sec] = 60 [min]

# Set the next condition (You only need to change the parameters that you want to change)
model.tdb = 30  # Change air temperature [°C]
model.tr = 35  # Change mean radiant temperature [°C]
# Run JOS-3 model
model.simulate(
    times=30,  # Number of loops of a simulation
    dtime=60,  # Time delta [sec]. The default is 60.
)  # Additional exposure time = 30 [loops] * 60 [sec] = 30 [min]

# Show the results
df = pd.DataFrame(model.dict_results())  # Make pandas.DataFrame
df[["t_skin_mean", "t_skin_head", "t_skin_chest", "t_skin_left_hand"]].plot()  # Plot time series of local skin temperature.
plt.legend(["Mean", "Head", "Chest", "Left hand"])  # Reset the legends
plt.ylabel("Skin temperature [°C]")  # Set y-label as 'Skin temperature [°C]'
plt.xlabel("Time [min]")  # Set x-label as 'Time [min]'
plt.savefig(os.path.join(jos3_example_directory, "jos3_example2_skin_temperatures.png"))  # Save plot at the current directory
plt.show()  # Show the plot

# Exporting the results as csv
model.to_csv(os.path.join(jos3_example_directory, "jos3_example2 (all output).csv"))
"""
# print(convert_and_print_local_clo_values_from_csv_to_dict(csv_name='local_clo_summary.csv'))
# print(add_prompt_to_code(sample_code))

# This dictionary contains the local and the whole body clothing insulation of typical clothing ensemble.
# It is based on the study by Juyoun et al. (https://escholarship.org/uc/item/18f0r375)
# and by Nomoto et al. (https://doi.org/10.1002/2475-8876.12124)
# Please note that value for the neck is the same as the measured value for the head
# and it does not take into account the insulation effect of the hair.
# Typically, the clothing insulation for the hair are quantified by assuming a head covering of approximately 0.6 to 1.0 clo.
local_clo_typical_ensembles = {
    "nude (mesh chair)": {
        "whole_body": 0.01,
        "local_body_part": {
            "head": 0.13,
            "neck": 0.13,
            "chest": 0.01,
            "back": 0.01,
            "pelvis": 0.04,
            "left_shoulder": 0.02,
            "left_arm": 0.0,
            "left_hand": 0.01,
            "right_shoulder": 0.02,
            "right_arm": 0.0,
            "right_hand": 0.01,
            "left_thigh": 0.01,
            "left_leg": 0.03,
            "left_foot": 0.05,
            "right_thigh": 0.01,
            "right_leg": 0.03,
            "right_foot": 0.05,
        },
    },
    "nude (nude chair)": {
        "whole_body": -0.02,
        "local_body_part": {
            "head": 0.13,
            "neck": 0.13,
            "chest": 0.05,
            "back": -0.14,
            "pelvis": -0.01,
            "left_shoulder": -0.01,
            "left_arm": -0.01,
            "left_hand": -0.02,
            "right_shoulder": -0.01,
            "right_arm": -0.01,
            "right_hand": -0.02,
            "left_thigh": -0.1,
            "left_leg": 0.0,
            "left_foot": 0.0,
            "right_thigh": -0.1,
            "right_leg": 0.0,
            "right_foot": 0.0,
        },
    },
    "panty": {
        "whole_body": 0.03,
        "local_body_part": {
            "head": 0.0,
            "neck": 0.0,
            "chest": 0.0,
            "back": 0.0,
            "pelvis": 0.24,
            "left_shoulder": 0.0,
            "left_arm": 0.0,
            "left_hand": 0.0,
            "right_shoulder": 0.0,
            "right_arm": 0.0,
            "right_hand": 0.0,
            "left_thigh": 0.05,
            "left_leg": 0.0,
            "left_foot": 0.05,
            "right_thigh": 0.05,
            "right_leg": 0.0,
            "right_foot": 0.05,
        },
    },
    "bra+panty": {
        "whole_body": 0.05,
        "local_body_part": {
            "head": 0.0,
            "neck": 0.0,
            "chest": 0.22,
            "back": 0.0,
            "pelvis": 0.18,
            "left_shoulder": 0.0,
            "left_arm": 0.0,
            "left_hand": 0.0,
            "right_shoulder": 0.0,
            "right_arm": 0.0,
            "right_hand": 0.0,
            "left_thigh": 0.03,
            "left_leg": 0.03,
            "left_foot": 0.08,
            "right_thigh": 0.03,
            "right_leg": 0.03,
            "right_foot": 0.08,
        },
    },
    "bra+panty, tanktop, shorts, sandals": {
        "whole_body": 0.22,
        "local_body_part": {
            "head": 0.0,
            "neck": 0.0,
            "chest": 0.57,
            "back": 0.27,
            "pelvis": 0.92,
            "left_shoulder": 0.04,
            "left_arm": 0.02,
            "left_hand": 0.02,
            "right_shoulder": 0.04,
            "right_arm": 0.02,
            "right_hand": 0.02,
            "left_thigh": 0.51,
            "left_leg": 0.01,
            "left_foot": 0.38,
            "right_thigh": 0.51,
            "right_leg": 0.01,
            "right_foot": 0.38,
        },
    },
    "bra+panty, long-sleeve shirt, shorts, sandals": {
        "whole_body": 0.43,
        "local_body_part": {
            "head": 0.0,
            "neck": 0.0,
            "chest": 1.43,
            "back": 1.02,
            "pelvis": 1.45,
            "left_shoulder": 0.29,
            "left_arm": 0.22,
            "left_hand": 0.01,
            "right_shoulder": 0.29,
            "right_arm": 0.22,
            "right_hand": 0.01,
            "left_thigh": 0.57,
            "left_leg": 0.01,
            "left_foot": 0.4,
            "right_thigh": 0.57,
            "right_leg": 0.01,
            "right_foot": 0.4,
        },
    },
    "bra+panty, sleeveless dress, sandals": {
        "whole_body": 0.29,
        "local_body_part": {
            "head": 0.0,
            "neck": 0.0,
            "chest": 0.85,
            "back": 0.48,
            "pelvis": 0.94,
            "left_shoulder": 0.0,
            "left_arm": 0.0,
            "left_hand": 0.0,
            "right_shoulder": 0.0,
            "right_arm": 0.0,
            "right_hand": 0.0,
            "left_thigh": 0.72,
            "left_leg": 0.0,
            "left_foot": 0.41,
            "right_thigh": 0.72,
            "right_leg": 0.0,
            "right_foot": 0.41,
        },
    },
    "bra+panty, T-shirt, long pants, socks, sneakers": {
        "whole_body": 0.52,
        "local_body_part": {
            "head": 0.0,
            "neck": 0.0,
            "chest": 1.14,
            "back": 0.84,
            "pelvis": 1.04,
            "left_shoulder": 0.42,
            "left_arm": 0.0,
            "left_hand": 0.0,
            "right_shoulder": 0.42,
            "right_arm": 0.0,
            "right_hand": 0.0,
            "left_thigh": 0.58,
            "left_leg": 0.62,
            "left_foot": 0.82,
            "right_thigh": 0.58,
            "right_leg": 0.62,
            "right_foot": 0.82,
        },
    },
    "bra+panty, sleeveless dress, cardigan, sandals": {
        "whole_body": 0.53,
        "local_body_part": {
            "head": 0.0,
            "neck": 0.0,
            "chest": 1.78,
            "back": 1.42,
            "pelvis": 1.19,
            "left_shoulder": 0.65,
            "left_arm": 0.41,
            "left_hand": 0.05,
            "right_shoulder": 0.65,
            "right_arm": 0.41,
            "right_hand": 0.05,
            "left_thigh": 0.77,
            "left_leg": 0.0,
            "left_foot": 0.39,
            "right_thigh": 0.77,
            "right_leg": 0.0,
            "right_foot": 0.39,
        },
    },
    "bra+panty, song-sleeve dress, socks, sneakers": {
        "whole_body": 0.54,
        "local_body_part": {
            "head": 0.0,
            "neck": 0.0,
            "chest": 1.49,
            "back": 1.1,
            "pelvis": 0.91,
            "left_shoulder": 0.72,
            "left_arm": 0.58,
            "left_hand": 0.03,
            "right_shoulder": 0.72,
            "right_arm": 0.58,
            "right_hand": 0.03,
            "left_thigh": 0.73,
            "left_leg": 0.07,
            "left_foot": 0.77,
            "right_thigh": 0.73,
            "right_leg": 0.07,
            "right_foot": 0.77,
        },
    },
    "bra+panty, long-sleeve dress, cardigan, socks, sneakers": {
        "whole_body": 0.67,
        "local_body_part": {
            "head": 0.0,
            "neck": 0.0,
            "chest": 2.05,
            "back": 1.32,
            "pelvis": 1.39,
            "left_shoulder": 1.14,
            "left_arm": 0.63,
            "left_hand": 0.04,
            "right_shoulder": 1.14,
            "right_arm": 0.63,
            "right_hand": 0.04,
            "left_thigh": 0.84,
            "left_leg": 0.05,
            "left_foot": 0.78,
            "right_thigh": 0.84,
            "right_leg": 0.05,
            "right_foot": 0.78,
        },
    },
    "bra+panty, tank top, skirt, sandals": {
        "whole_body": 0.31,
        "local_body_part": {
            "head": 0.0,
            "neck": 0.0,
            "chest": 0.83,
            "back": 0.22,
            "pelvis": 0.99,
            "left_shoulder": 0.0,
            "left_arm": 0.0,
            "left_hand": 0.03,
            "right_shoulder": 0.0,
            "right_arm": 0.0,
            "right_hand": 0.03,
            "left_thigh": 0.88,
            "left_leg": 0.05,
            "left_foot": 0.44,
            "right_thigh": 0.88,
            "right_leg": 0.05,
            "right_foot": 0.44,
        },
    },
    "bra+panty, long sleeve shirts, skirt, sandals": {
        "whole_body": 0.52,
        "local_body_part": {
            "head": 0.0,
            "neck": 0.0,
            "chest": 1.62,
            "back": 0.99,
            "pelvis": 1.41,
            "left_shoulder": 0.31,
            "left_arm": 0.28,
            "left_hand": 0.03,
            "right_shoulder": 0.31,
            "right_arm": 0.28,
            "right_hand": 0.03,
            "left_thigh": 0.82,
            "left_leg": 0.04,
            "left_foot": 0.41,
            "right_thigh": 0.82,
            "right_leg": 0.04,
            "right_foot": 0.41,
        },
    },
    "bra+panty, dress shirts, skirt, stocking, formal shoes": {
        "whole_body": 0.62,
        "local_body_part": {
            "head": 0.0,
            "neck": 0.0,
            "chest": 1.58,
            "back": 0.99,
            "pelvis": 1.31,
            "left_shoulder": 0.91,
            "left_arm": 0.64,
            "left_hand": 0.04,
            "right_shoulder": 0.91,
            "right_arm": 0.64,
            "right_hand": 0.04,
            "left_thigh": 0.87,
            "left_leg": 0.05,
            "left_foot": 0.81,
            "right_thigh": 0.87,
            "right_leg": 0.05,
            "right_foot": 0.81,
        },
    },
    "bra+panty, dress shirts, skirt, leggings, sandals": {
        "whole_body": 0.65,
        "local_body_part": {
            "head": 0.13,
            "neck": 0.13,
            "chest": 1.59,
            "back": 1.04,
            "pelvis": 1.36,
            "left_shoulder": 0.91,
            "left_arm": 0.67,
            "left_hand": 0.07,
            "right_shoulder": 0.91,
            "right_arm": 0.67,
            "right_hand": 0.07,
            "left_thigh": 1.26,
            "left_leg": 0.12,
            "left_foot": 0.43,
            "right_thigh": 1.26,
            "right_leg": 0.12,
            "right_foot": 0.43,
        },
    },
    "bra+panty, thin dress shirts, long pants, socks, sneakers": {
        "whole_body": 0.82,
        "local_body_part": {
            "head": 0.0,
            "neck": 0.0,
            "chest": 3.35,
            "back": 1.73,
            "pelvis": 1.63,
            "left_shoulder": 1.99,
            "left_arm": 1.49,
            "left_hand": 0.11,
            "right_shoulder": 1.99,
            "right_arm": 1.49,
            "right_hand": 0.11,
            "left_thigh": 0.6,
            "left_leg": 0.43,
            "left_foot": 0.68,
            "right_thigh": 0.6,
            "right_leg": 0.43,
            "right_foot": 0.68,
        },
    },
    "bra+panty, long sleeve shirts, long pants, socks, sneakers": {
        "whole_body": 0.8,
        "local_body_part": {
            "head": 0.0,
            "neck": 0.0,
            "chest": 2.47,
            "back": 1.48,
            "pelvis": 1.58,
            "left_shoulder": 0.98,
            "left_arm": 0.58,
            "left_hand": 0.04,
            "right_shoulder": 0.98,
            "right_arm": 0.58,
            "right_hand": 0.04,
            "left_thigh": 0.69,
            "left_leg": 0.65,
            "left_foot": 0.89,
            "right_thigh": 0.69,
            "right_leg": 0.65,
            "right_foot": 0.89,
        },
    },
    "bra+panty, T-shirt, long sleeve shirts, long pants, socks, sneakers": {
        "whole_body": 0.83,
        "local_body_part": {
            "head": 0.25,
            "neck": 0.25,
            "chest": 3.88,
            "back": 2.28,
            "pelvis": 2.07,
            "left_shoulder": 1.89,
            "left_arm": 1.41,
            "left_hand": 0.16,
            "right_shoulder": 1.89,
            "right_arm": 1.41,
            "right_hand": 0.16,
            "left_thigh": 0.83,
            "left_leg": 0.66,
            "left_foot": 0.86,
            "right_thigh": 0.83,
            "right_leg": 0.66,
            "right_foot": 0.86,
        },
    },
    "bra+panty, T-shirt, jeans, socks, sneakers": {
        "whole_body": 0.57,
        "local_body_part": {
            "head": 0.0,
            "neck": 0.0,
            "chest": 1.29,
            "back": 0.93,
            "pelvis": 1.3,
            "left_shoulder": 0.68,
            "left_arm": 0.0,
            "left_hand": 0.0,
            "right_shoulder": 0.68,
            "right_arm": 0.0,
            "right_hand": 0.0,
            "left_thigh": 0.65,
            "left_leg": 0.47,
            "left_foot": 0.73,
            "right_thigh": 0.65,
            "right_leg": 0.47,
            "right_foot": 0.73,
        },
    },
    "bra+panty, long sleeve shirts, jeans, socks, sneakers": {
        "whole_body": 0.74,
        "local_body_part": {
            "head": 0.0,
            "neck": 0.0,
            "chest": 1.58,
            "back": 0.98,
            "pelvis": 1.35,
            "left_shoulder": 0.86,
            "left_arm": 0.71,
            "left_hand": 0.07,
            "right_shoulder": 0.86,
            "right_arm": 0.71,
            "right_hand": 0.07,
            "left_thigh": 0.74,
            "left_leg": 0.48,
            "left_foot": 0.74,
            "right_thigh": 0.74,
            "right_leg": 0.48,
            "right_foot": 0.74,
        },
    },
    "bra+panty, oxford shirts, long thin pants, socks, sneakers": {
        "whole_body": 0.83,
        "local_body_part": {
            "head": 0.16,
            "neck": 0.16,
            "chest": 1.39,
            "back": 1.02,
            "pelvis": 1.34,
            "left_shoulder": 0.83,
            "left_arm": 0.69,
            "left_hand": 0.22,
            "right_shoulder": 0.83,
            "right_arm": 0.69,
            "right_hand": 0.22,
            "left_thigh": 1.02,
            "left_leg": 0.68,
            "left_foot": 0.8,
            "right_thigh": 1.02,
            "right_leg": 0.68,
            "right_foot": 0.8,
        },
    },
    "bra+panty, thin dress shirts (roll-up), long pants, socks, sneakers": {
        "whole_body": 0.81,
        "local_body_part": {
            "head": 0.0,
            "neck": 0.0,
            "chest": 3.6,
            "back": 1.83,
            "pelvis": 1.71,
            "left_shoulder": 2.16,
            "left_arm": 1.49,
            "left_hand": 0.13,
            "right_shoulder": 2.16,
            "right_arm": 1.49,
            "right_hand": 0.13,
            "left_thigh": 0.64,
            "left_leg": 0.43,
            "left_foot": 0.69,
            "right_thigh": 0.64,
            "right_leg": 0.43,
            "right_foot": 0.69,
        },
    },
    "bra+panty, T-shirt, short sleeve shirt, long pants, socks, sneakers": {
        "whole_body": 0.71,
        "local_body_part": {
            "head": 0.12,
            "neck": 0.12,
            "chest": 2.15,
            "back": 1.4,
            "pelvis": 1.71,
            "left_shoulder": 1.22,
            "left_arm": 0.02,
            "left_hand": 0.05,
            "right_shoulder": 1.22,
            "right_arm": 0.02,
            "right_hand": 0.05,
            "left_thigh": 0.79,
            "left_leg": 0.48,
            "left_foot": 0.67,
            "right_thigh": 0.79,
            "right_leg": 0.48,
            "right_foot": 0.67,
        },
    },
    "bra+panty, sports shirts, long pants, socks, sneakers": {
        "whole_body": 0.8,
        "local_body_part": {
            "head": 0.05,
            "neck": 0.05,
            "chest": 1.92,
            "back": 1.31,
            "pelvis": 1.41,
            "left_shoulder": 1.14,
            "left_arm": 0.86,
            "left_hand": 0.18,
            "right_shoulder": 1.14,
            "right_arm": 0.86,
            "right_hand": 0.18,
            "left_thigh": 0.59,
            "left_leg": 0.49,
            "left_foot": 0.75,
            "right_thigh": 0.59,
            "right_leg": 0.49,
            "right_foot": 0.75,
        },
    },
    "bra+panty, sports shirts, sports pants, sports socks, sports shoes": {
        "whole_body": 0.87,
        "local_body_part": {
            "head": 0.07,
            "neck": 0.07,
            "chest": 1.87,
            "back": 1.17,
            "pelvis": 1.26,
            "left_shoulder": 1.2,
            "left_arm": 1.07,
            "left_hand": 0.09,
            "right_shoulder": 1.2,
            "right_arm": 1.07,
            "right_hand": 0.09,
            "left_thigh": 0.62,
            "left_leg": 0.77,
            "left_foot": 1.58,
            "right_thigh": 0.62,
            "right_leg": 0.77,
            "right_foot": 1.58,
        },
    },
    "bra+panty, thin dress shirts, long pants, wool sweater, socks, sneakers": {
        "whole_body": 0.92,
        "local_body_part": {
            "head": 0.09,
            "neck": 0.09,
            "chest": 2.39,
            "back": 1.64,
            "pelvis": 1.71,
            "left_shoulder": 1.36,
            "left_arm": 1.29,
            "left_hand": 0.21,
            "right_shoulder": 1.36,
            "right_arm": 1.29,
            "right_hand": 0.21,
            "left_thigh": 0.7,
            "left_leg": 0.52,
            "left_foot": 0.77,
            "right_thigh": 0.7,
            "right_leg": 0.52,
            "right_foot": 0.77,
        },
    },
    "bra+panty, thin dress shirts, long pants, cashmere sweater, socks, sneakers": {
        "whole_body": 0.87,
        "local_body_part": {
            "head": 0.1,
            "neck": 0.1,
            "chest": 2.4,
            "back": 1.72,
            "pelvis": 1.67,
            "left_shoulder": 1.33,
            "left_arm": 1.23,
            "left_hand": 0.08,
            "right_shoulder": 1.33,
            "right_arm": 1.23,
            "right_hand": 0.08,
            "left_thigh": 0.61,
            "left_leg": 0.47,
            "left_foot": 0.77,
            "right_thigh": 0.61,
            "right_leg": 0.47,
            "right_foot": 0.77,
        },
    },
    "bra+panty, T-shirt, long sleeve shirts, long pants, winter jacket, socks, sneakers": {
        "whole_body": 1.18,
        "local_body_part": {
            "head": 0.65,
            "neck": 0.65,
            "chest": 5.26,
            "back": 3.07,
            "pelvis": 2.2,
            "left_shoulder": 3.14,
            "left_arm": 2.07,
            "left_hand": 0.08,
            "right_shoulder": 3.14,
            "right_arm": 2.07,
            "right_hand": 0.08,
            "left_thigh": 0.67,
            "left_leg": 0.54,
            "left_foot": 0.77,
            "right_thigh": 0.67,
            "right_leg": 0.54,
            "right_foot": 0.77,
        },
    },
    "bra+panty, T-shirt, long sleeve shirts, jeans, sports jumper, socks, sneakers": {
        "whole_body": 1.07,
        "local_body_part": {
            "head": 0.28,
            "neck": 0.28,
            "chest": 3.99,
            "back": 2.12,
            "pelvis": 2.0,
            "left_shoulder": 1.7,
            "left_arm": 1.36,
            "left_hand": 0.1,
            "right_shoulder": 1.7,
            "right_arm": 1.36,
            "right_hand": 0.1,
            "left_thigh": 0.92,
            "left_leg": 0.48,
            "left_foot": 1.07,
            "right_thigh": 0.92,
            "right_leg": 0.48,
            "right_foot": 1.07,
        },
    },
    "bra+panty, T-shirt, long sleeve shirts, long pants, ventura jacket, socks, sneakers": {
        "whole_body": 0.9,
        "local_body_part": {
            "head": 0.09,
            "neck": 0.09,
            "chest": 2.66,
            "back": 1.42,
            "pelvis": 1.57,
            "left_shoulder": 1.32,
            "left_arm": 0.99,
            "left_hand": 0.14,
            "right_shoulder": 1.32,
            "right_arm": 0.99,
            "right_hand": 0.14,
            "left_thigh": 0.73,
            "left_leg": 0.66,
            "left_foot": 0.85,
            "right_thigh": 0.73,
            "right_leg": 0.66,
            "right_foot": 0.85,
        },
    },
    "bra+panty, turtle neck, long pants, short trench coat, socks, sneakers": {
        "whole_body": 1.24,
        "local_body_part": {
            "head": 0.06,
            "neck": 0.06,
            "chest": 3.22,
            "back": 1.99,
            "pelvis": 2.03,
            "left_shoulder": 1.62,
            "left_arm": 1.5,
            "left_hand": 0.37,
            "right_shoulder": 1.62,
            "right_arm": 1.5,
            "right_hand": 0.37,
            "left_thigh": 1.51,
            "left_leg": 0.65,
            "left_foot": 0.8,
            "right_thigh": 1.51,
            "right_leg": 0.65,
            "right_foot": 0.8,
        },
    },
    "bra+panty, tank top, long sleeve shirts, blazer, skirt, sandals": {
        "whole_body": 0.86,
        "local_body_part": {
            "head": 0.0,
            "neck": 0.0,
            "chest": 3.24,
            "back": 1.81,
            "pelvis": 2.06,
            "left_shoulder": 1.98,
            "left_arm": 1.13,
            "left_hand": 0.07,
            "right_shoulder": 1.98,
            "right_arm": 1.13,
            "right_hand": 0.07,
            "left_thigh": 1.19,
            "left_leg": 0.04,
            "left_foot": 0.44,
            "right_thigh": 1.19,
            "right_leg": 0.04,
            "right_foot": 0.44,
        },
    },
    "bra+panty, long sleeve shirts, wool skirt, socks, formal shoes": {
        "whole_body": 0.59,
        "local_body_part": {
            "head": 0.0,
            "neck": 0.0,
            "chest": 1.21,
            "back": 0.74,
            "pelvis": 1.56,
            "left_shoulder": 0.44,
            "left_arm": 0.24,
            "left_hand": 0.17,
            "right_shoulder": 0.44,
            "right_arm": 0.24,
            "right_hand": 0.17,
            "left_thigh": 1.52,
            "left_leg": 0.09,
            "left_foot": 0.74,
            "right_thigh": 1.52,
            "right_leg": 0.09,
            "right_foot": 0.74,
        },
    },
    "bra+panty, turtleneck, wool skirt, socks, formal shoes": {
        "whole_body": 0.7,
        "local_body_part": {
            "head": 0.0,
            "neck": 0.0,
            "chest": 1.11,
            "back": 0.94,
            "pelvis": 1.52,
            "left_shoulder": 0.73,
            "left_arm": 0.62,
            "left_hand": 0.14,
            "right_shoulder": 0.73,
            "right_arm": 0.62,
            "right_hand": 0.14,
            "left_thigh": 1.53,
            "left_leg": 0.09,
            "left_foot": 0.85,
            "right_thigh": 1.53,
            "right_leg": 0.09,
            "right_foot": 0.85,
        },
    },
    "bra+panty, long sleeve shirt, wool skirt, sweater, socks, formal shoes": {
        "whole_body": 0.91,
        "local_body_part": {
            "head": 0.14,
            "neck": 0.14,
            "chest": 2.82,
            "back": 1.53,
            "pelvis": 1.79,
            "left_shoulder": 1.22,
            "left_arm": 0.97,
            "left_hand": 0.08,
            "right_shoulder": 1.22,
            "right_arm": 0.97,
            "right_hand": 0.08,
            "left_thigh": 1.53,
            "left_leg": 0.11,
            "left_foot": 0.83,
            "right_thigh": 1.53,
            "right_leg": 0.11,
            "right_foot": 0.83,
        },
    },
    "bra+panty, thin dress shirts, slacks, tie, socks, sneakers": {
        "whole_body": 0.57,
        "local_body_part": {
            "head": 0.0,
            "neck": 0.0,
            "chest": 1.69,
            "back": 0.8,
            "pelvis": 1.08,
            "left_shoulder": 0.67,
            "left_arm": 0.58,
            "left_hand": 0.07,
            "right_shoulder": 0.67,
            "right_arm": 0.58,
            "right_hand": 0.07,
            "left_thigh": 0.36,
            "left_leg": 0.39,
            "left_foot": 0.74,
            "right_thigh": 0.36,
            "right_leg": 0.39,
            "right_foot": 0.74,
        },
    },
    "bra+panty, thin dress shirts, slacks, blazer, tie, belt, socks, formal shoes": {
        "whole_body": 0.93,
        "local_body_part": {
            "head": 0.0,
            "neck": 0.0,
            "chest": 3.6,
            "back": 1.83,
            "pelvis": 1.71,
            "left_shoulder": 2.16,
            "left_arm": 1.49,
            "left_hand": 0.13,
            "right_shoulder": 2.16,
            "right_arm": 1.49,
            "right_hand": 0.13,
            "left_thigh": 0.64,
            "left_leg": 0.43,
            "left_foot": 0.69,
            "right_thigh": 0.64,
            "right_leg": 0.43,
            "right_foot": 0.69,
        },
    },
    "bra+panty, long sleeve shirts, long pants, blazer, socks, sneakers": {
        "whole_body": 0.96,
        "local_body_part": {
            "head": 0.04,
            "neck": 0.04,
            "chest": 3.3,
            "back": 1.67,
            "pelvis": 2.2,
            "left_shoulder": 2.1,
            "left_arm": 1.43,
            "left_hand": 0.09,
            "right_shoulder": 2.1,
            "right_arm": 1.43,
            "right_hand": 0.09,
            "left_thigh": 0.72,
            "left_leg": 0.42,
            "left_foot": 0.67,
            "right_thigh": 0.72,
            "right_leg": 0.42,
            "right_foot": 0.67,
        },
    },
    "bra+panty, T-shirt, long sleeve shirts, long pants, winter jacket (Notica)": {
        "whole_body": 1.05,
        "local_body_part": {
            "head": 0.04,
            "neck": 0.04,
            "chest": 3.88,
            "back": 2.26,
            "pelvis": 1.97,
            "left_shoulder": 1.82,
            "left_arm": 1.46,
            "left_hand": 0.17,
            "right_shoulder": 1.82,
            "right_arm": 1.46,
            "right_hand": 0.17,
            "left_thigh": 0.81,
            "left_leg": 0.57,
            "left_foot": 0.78,
            "right_thigh": 0.81,
            "right_leg": 0.57,
            "right_foot": 0.78,
        },
    },
    "bra+panty, turtle neck, ski-jumper, skin pants, sports socks, sports shoes": {
        "whole_body": 1.84,
        "local_body_part": {
            "head": 0.89,
            "neck": 0.89,
            "chest": 5.24,
            "back": 2.87,
            "pelvis": 2.64,
            "left_shoulder": 2.55,
            "left_arm": 2.16,
            "left_hand": 0.46,
            "right_shoulder": 2.55,
            "right_arm": 2.16,
            "right_hand": 0.46,
            "left_thigh": 1.49,
            "left_leg": 1.82,
            "left_foot": 1.56,
            "right_thigh": 1.49,
            "right_leg": 1.82,
            "right_foot": 1.56,
        },
    },
    "bra+panty, turtle neck, ski-jumper and hood, skin pants, sports socks, sports shoes": {
        "whole_body": 1.87,
        "local_body_part": {
            "head": 1.63,
            "neck": 1.63,
            "chest": 5.12,
            "back": 2.7,
            "pelvis": 2.57,
            "left_shoulder": 2.58,
            "left_arm": 2.16,
            "left_hand": 0.49,
            "right_shoulder": 2.58,
            "right_arm": 2.16,
            "right_hand": 0.49,
            "left_thigh": 1.44,
            "left_leg": 1.76,
            "left_foot": 1.54,
            "right_thigh": 1.44,
            "right_leg": 1.76,
            "right_foot": 1.54,
        },
    },
    "bra+panty, turtle neck, goose down, ski pants, sports socks, sports shoes": {
        "whole_body": 2.53,
        "local_body_part": {
            "head": 1.17,
            "neck": 1.17,
            "chest": 15.44,
            "back": 5.5,
            "pelvis": 5.2,
            "left_shoulder": 6.55,
            "left_arm": 5.58,
            "left_hand": 0.35,
            "right_shoulder": 6.55,
            "right_arm": 5.58,
            "right_hand": 0.35,
            "left_thigh": 2.12,
            "left_leg": 1.7,
            "left_foot": 1.54,
            "right_thigh": 2.12,
            "right_leg": 1.7,
            "right_foot": 1.54,
        },
    },
    "bra+panty, turtle neck, goose down-with hood, ski pants, sports socks, sports shoes": {
        "whole_body": 2.75,
        "local_body_part": {
            "head": 3.52,
            "neck": 3.52,
            "chest": 12.62,
            "back": 3.99,
            "pelvis": 5.05,
            "left_shoulder": 6.2,
            "left_arm": 5.73,
            "left_hand": 0.53,
            "right_shoulder": 6.2,
            "right_arm": 5.73,
            "right_hand": 0.53,
            "left_thigh": 2.11,
            "left_leg": 1.81,
            "left_foot": 1.58,
            "right_thigh": 2.11,
            "right_leg": 1.81,
            "right_foot": 1.58,
        },
    },
    "bra+panty, turtle neck, goose down-with hood and gloves, ski pants, sports socks, sports shoes": {
        "whole_body": 3.27,
        "local_body_part": {
            "head": 3.92,
            "neck": 3.92,
            "chest": 16.13,
            "back": 4.47,
            "pelvis": 5.71,
            "left_shoulder": 7.12,
            "left_arm": 5.37,
            "left_hand": 2.54,
            "right_shoulder": 7.12,
            "right_arm": 5.37,
            "right_hand": 2.54,
            "left_thigh": 2.14,
            "left_leg": 1.82,
            "left_foot": 1.61,
            "right_thigh": 2.14,
            "right_leg": 1.82,
            "right_foot": 1.61,
        },
    },
    "briefs, socks, T-shirt, half pants, sneakers": {
        "whole_body": 0.53,
        "local_body_part": {
            "head": 0.0,
            "neck": 0.0,
            "chest": 0.5,
            "back": 1.13,
            "pelvis": 1.21,
            "left_shoulder": 0.39,
            "left_arm": 0.0,
            "left_hand": 0.0,
            "right_shoulder": 0.39,
            "right_arm": 0.0,
            "right_hand": 0.0,
            "left_thigh": 0.94,
            "left_leg": 0.07,
            "left_foot": 0.62,
            "right_thigh": 0.94,
            "right_leg": 0.07,
            "right_foot": 0.62,
        },
    },
    "briefs, undershirt, sports t-shirts, sports shorts": {
        "whole_body": 0.7,
        "local_body_part": {
            "head": 0.0,
            "neck": 0.0,
            "chest": 0.9,
            "back": 1.76,
            "pelvis": 2.14,
            "left_shoulder": 0.48,
            "left_arm": 0.0,
            "left_hand": 0.0,
            "right_shoulder": 0.48,
            "right_arm": 0.0,
            "right_hand": 0.0,
            "left_thigh": 1.18,
            "left_leg": 0.0,
            "left_foot": 0.01,
            "right_thigh": 1.18,
            "right_leg": 0.0,
            "right_foot": 0.01,
        },
    },
    "briefs, socks, polo shirt, long pants, sneakers": {
        "whole_body": 0.54,
        "local_body_part": {
            "head": 0.02,
            "neck": 0.02,
            "chest": 0.52,
            "back": 0.97,
            "pelvis": 1.12,
            "left_shoulder": 0.34,
            "left_arm": 0.0,
            "left_hand": 0.0,
            "right_shoulder": 0.34,
            "right_arm": 0.0,
            "right_hand": 0.0,
            "left_thigh": 0.79,
            "left_leg": 0.56,
            "left_foot": 0.65,
            "right_thigh": 0.79,
            "right_leg": 0.56,
            "right_foot": 0.65,
        },
    },
    "briefs, under shirt, long-sleeved shirt, long pants": {
        "whole_body": 1.01,
        "local_body_part": {
            "head": 0.0,
            "neck": 0.0,
            "chest": 1.2,
            "back": 1.65,
            "pelvis": 2.29,
            "left_shoulder": 0.98,
            "left_arm": 0.78,
            "left_hand": 0.03,
            "right_shoulder": 0.98,
            "right_arm": 0.78,
            "right_hand": 0.03,
            "left_thigh": 1.46,
            "left_leg": 0.62,
            "left_foot": 0.02,
            "right_thigh": 1.46,
            "right_leg": 0.62,
            "right_foot": 0.02,
        },
    },
    "briefs, socks, undershirt, short-sleeved shirt, long pants, belt, shoes": {
        "whole_body": 0.72,
        "local_body_part": {
            "head": 0.01,
            "neck": 0.01,
            "chest": 0.8,
            "back": 1.4,
            "pelvis": 1.55,
            "left_shoulder": 0.54,
            "left_arm": 0.0,
            "left_hand": 0.0,
            "right_shoulder": 0.54,
            "right_arm": 0.0,
            "right_hand": 0.0,
            "left_thigh": 0.89,
            "left_leg": 0.64,
            "left_foot": 0.99,
            "right_thigh": 0.89,
            "right_leg": 0.64,
            "right_foot": 0.99,
        },
    },
    "briefs, socks, undershirt, long-sleeved shirt, long pants, belt": {
        "whole_body": 0.77,
        "local_body_part": {
            "head": 0.0,
            "neck": 0.0,
            "chest": 0.86,
            "back": 1.45,
            "pelvis": 1.54,
            "left_shoulder": 0.82,
            "left_arm": 0.6,
            "left_hand": 0.01,
            "right_shoulder": 0.82,
            "right_arm": 0.6,
            "right_hand": 0.01,
            "left_thigh": 0.9,
            "left_leg": 0.66,
            "left_foot": 0.64,
            "right_thigh": 0.9,
            "right_leg": 0.66,
            "right_foot": 0.64,
        },
    },
    "briefs, socks, undershirt, long-sleeved shirt, jacket, long pants, belt, shoes": {
        "whole_body": 1.39,
        "local_body_part": {
            "head": 0.02,
            "neck": 0.02,
            "chest": 2.13,
            "back": 2.28,
            "pelvis": 3.04,
            "left_shoulder": 1.8,
            "left_arm": 1.54,
            "left_hand": 0.15,
            "right_shoulder": 1.8,
            "right_arm": 1.54,
            "right_hand": 0.15,
            "left_thigh": 1.33,
            "left_leg": 0.69,
            "left_foot": 0.97,
            "right_thigh": 1.33,
            "right_leg": 0.69,
            "right_foot": 0.97,
        },
    },
    "briefs, socks, undershirt, work jacket, work pants, safety shoes": {
        "whole_body": 0.8,
        "local_body_part": {
            "head": 0.0,
            "neck": 0.0,
            "chest": 1.25,
            "back": 1.39,
            "pelvis": 1.78,
            "left_shoulder": 0.84,
            "left_arm": 0.71,
            "left_hand": 0.08,
            "right_shoulder": 0.84,
            "right_arm": 0.71,
            "right_hand": 0.08,
            "left_thigh": 0.65,
            "left_leg": 0.59,
            "left_foot": 1.12,
            "right_thigh": 0.65,
            "right_leg": 0.59,
            "right_foot": 1.12,
        },
    },
}



from typing import Union, List

import numpy as np

from pythermalcomfort.models import pmv_ppd


def pmv(
    tdb: Union[float, int, np.ndarray, List[float], List[int]],
    tr: Union[float, int, np.ndarray, List[float], List[int]],
    vr: Union[float, int, np.ndarray, List[float], List[int]],
    rh: Union[float, int, np.ndarray, List[float], List[int]],
    met: Union[float, int, np.ndarray, List[float], List[int]],
    clo: Union[float, int, np.ndarray, List[float], List[int]],
    wme: Union[float, int, np.ndarray, List[float], List[int]] = 0,
    standard="ISO",
    units="SI",
    limit_inputs=True,
    airspeed_control=True,
):
    """Returns Predicted Mean Vote (`PMV`_) calculated in accordance to main
    thermal comfort Standards. The PMV is an index that predicts the mean value
    of the thermal sensation votes (self-reported perceptions) of a large group
    of people on a sensation scale expressed from –3 to +3 corresponding to the
    categories: cold, cool, slightly cool, neutral, slightly warm, warm, and hot. [1]_

    While the PMV equation is the same for both the ISO and ASHRAE standards, in the
    ASHRAE 55 PMV equation, the SET is used to calculate the cooling effect first,
    this is then subtracted from both the air and mean radiant temperatures, and the
    differences are used as input to the PMV model, while the airspeed is set to 0.1m/s.
    Please read more in the Note below.

    Parameters
    ----------
    tdb : float, int, or array-like
        dry bulb air temperature, default in [°C] in [°F] if `units` = 'IP'
    tr : float, int, or array-like
        mean radiant temperature, default in [°C] in [°F] if `units` = 'IP'
    vr : float, int, or array-like
        relative air speed, default in [m/s] in [fps] if `units` = 'IP'

        Note: vr is the relative air speed caused by body movement and not the air
        speed measured by the air speed sensor. The relative air speed is the sum of the
        average air speed measured by the sensor plus the activity-generated air speed
        (Vag). Where Vag is the activity-generated air speed caused by motion of
        individual body parts. vr can be calculated using the function
        :py:meth:`pythermalcomfort.utilities.v_relative`.
    rh : float, int, or array-like
        relative humidity, [%]
    met : float, int, or array-like
        metabolic rate, [met]
    clo : float, int, or array-like
        clothing insulation, [clo]

        Note: The activity as well as the air speed modify the insulation characteristics
        of the clothing and the adjacent air layer. Consequently, the ISO 7730 states that
        the clothing insulation shall be corrected [2]_. The ASHRAE 55 Standard corrects
        for the effect of the body movement for met equal or higher than 1.2 met using
        the equation clo = Icl × (0.6 + 0.4/met) The dynamic clothing insulation, clo,
        can be calculated using the function
        :py:meth:`pythermalcomfort.utilities.clo_dynamic`.
    wme : float, int, or array-like
        external work, [met] default 0
    standard : str, optional
        select comfort standard used for calculation.
        Supported values are 'ASHRAE' and 'ISO'. Defaults to 'ISO'.

        - If "ISO", then the ISO Equation is used
        - If "ASHRAE", then the ASHRAE Equation is used

        Note: While the PMV equation is the same for both the ISO and ASHRAE standards,
        the ASHRAE Standard Use of the PMV model is limited to air speeds below 0.10
        m/s (20 fpm).
        When air speeds exceed 0.10 m/s (20 fpm), the comfort zone boundaries are
        adjusted based on the SET model.
        This change was indroduced by the `Addendum C to Standard 55-2020`_
    units : str, optional
        select the SI (International System of Units) or the IP (Imperial Units) system.
        Supported values are 'SI' and 'IP'. Defaults to 'SI'.
    limit_inputs : boolean default True
        By default, if the inputs are outsude the standard applicability limits the
        function returns nan. If False returns pmv and ppd values even if input values are
        outside the applicability limits of the model.

        The ASHRAE 55 2020 limits are 10 < tdb [°C] < 40, 10 < tr [°C] < 40,
        0 < vr [m/s] < 2, 1 < met [met] < 4, and 0 < clo [clo] < 1.5.
        The ISO 7730 2005 limits are 10 < tdb [°C] < 30, 10 < tr [°C] < 40,
        0 < vr [m/s] < 1, 0.8 < met [met] < 4, 0 < clo [clo] < 2, and -2 < PMV < 2.
    airspeed_control : boolean default True
        This only applies if standard = "ASHRAE". By default it is assumed that the
        occupant has control over the airspeed. In this case the ASHRAE 55 Standard does
        not impose any airspeed limits. On the other hand, if the occupant has no control
        over the airspeed the ASHRAE 55 imposes an upper limit for v which varies as a
        function of the operative temperature, for more information please consult the
        Standard.

    Returns
    -------
    pmv : float, int, or array-like
        Predicted Mean Vote

    Notes
    -----
    You can use this function to calculate the `PMV`_ [1]_ [2]_.

    .. _PMV: https://en.wikipedia.org/wiki/Thermal_comfort#PMV/PPD_method
    .. _Addendum C to Standard 55-2020: https://www.ashrae.org/file%20library/technical%20resources/standards%20and%20guidelines/standards%20addenda/55_2020_c_20210430.pdf

    Examples
    --------
    .. code-block:: python

        >>> from pythermalcomfort.models import pmv
        >>> from pythermalcomfort.utilities import v_relative, clo_dynamic
        >>> t_db = 25
        >>> t_r = 25
        >>> relative_humidity = 50
        >>> v = 0.1
        >>> met_rate = 1.4
        >>> clo_insulation = 0.5
        >>> # calculate relative air speed
        >>> v_r = v_relative(v=v, met=met_rate)
        >>> # calculate dynamic clothing
        >>> clo_d = clo_dynamic(clo=clo_insulation, met=met_rate)
        >>> results = pmv(tdb=t_db, tr=t_r, vr=v_r, rh=relative_humidity, met=met_rate, clo=clo_d)
        >>> print(results)
        0.06
        >>> # you can also pass an array-like of inputs
        >>> results = pmv(tdb=[22, 25], tr=tr, vr=v_r, rh=rh, met=met, clo=clo_d)
        >>> print(results)
        array([-0.47,  0.06])
    """

    return pmv_ppd(
        tdb,
        tr,
        vr,
        rh,
        met,
        clo,
        wme,
        standard=standard,
        units=units,
        limit_inputs=limit_inputs,
        airspeed_control=airspeed_control,
    )["pmv"]



from pythermalcomfort.models import wc
import pytest


class TestWc:

    #  Calculates the wind chill index (WCI) for given dry bulb air temperature and wind speed
    def test_calculates_wci(self):
        assert wc(tdb=0, v=0.1) == {"wci": 518.6}
        assert wc(tdb=0, v=1.5) == {"wci": 813.5}
        assert wc(tdb=-5, v=5.5) == {"wci": 1255.2}
        assert wc(tdb=-10, v=11) == {"wci": 1631.1}
        assert wc(tdb=-5, v=11) == {"wci": 1441.4}

    #  Returns the WCI value in a dictionary format
    def test_returns_wci_dictionary(self):
        assert isinstance(wc(tdb=0, v=0.1), dict)
        assert isinstance(wc(tdb=0, v=1.5), dict)
        assert isinstance(wc(tdb=-5, v=5.5), dict)
        assert isinstance(wc(tdb=-10, v=11), dict)
        assert isinstance(wc(tdb=-5, v=11), dict)

    #  Rounds the output value if round=True
    def test_rounds_output_value(self):
        assert wc(tdb=0, v=0.1, round=True) == {"wci": 518.6}
        result = wc(tdb=0, v=0.1, round=False)
        assert abs(result["wci"] - 518.587) < 0.01

    #  Raises TypeError if tdb parameter is not provided
    def test_raises_type_error_tdb_not_provided(self):
        with pytest.raises(TypeError):
            wc(v=0.1)

    #  Raises TypeError if v parameter is not provided
    def test_raises_type_error_v_not_provided(self):
        with pytest.raises(TypeError):
            wc(tdb=0)

    #  Raises TypeError if tdb parameter is not a float
    def test_raises_type_error_tdb_not_float(self):
        with pytest.raises(TypeError):
            wc(tdb="0", v=0.1)



import json

import numpy as np
import requests
import math
import pytest

from pythermalcomfort.models import (
    pmv_ppd,
)
from pythermalcomfort.models.pmv_ppd import _pmv_ppd_optimized

# get file containing validation tables
url = "https://raw.githubusercontent.com/FedericoTartarini/validation-data-comfort-models/main/validation_data.json"
resp = requests.get(url)
reference_tables = json.loads(resp.text)

# fmt: off
data_test_set_ip = [  # I have commented the lines of code that don't pass the test
    {'tdb': 77, 'tr': 77, 'v': 29.5 / 60, 'rh': 50, 'met': 1, 'clo': 0.5, 'set': 74.9},
    {'tdb': 32, 'tr': 77, 'v': 29.5 / 60, 'rh': 50, 'met': 1, 'clo': 0.5, 'set': 53.7},
    {'tdb': 50, 'tr': 77, 'v': 29.5 / 60, 'rh': 50, 'met': 1, 'clo': 0.5, 'set': 62.3},
    {'tdb': 59, 'tr': 77, 'v': 29.5 / 60, 'rh': 50, 'met': 1, 'clo': 0.5, 'set': 66.5},
    {'tdb': 68, 'tr': 77, 'v': 29.5 / 60, 'rh': 50, 'met': 1, 'clo': 0.5, 'set': 70.7},
    {'tdb': 86, 'tr': 77, 'v': 29.5 / 60, 'rh': 50, 'met': 1, 'clo': 0.5, 'set': 79.6},
    {'tdb': 104, 'tr': 77, 'v': 29.5 / 60, 'rh': 50, 'met': 1, 'clo': 0.5, 'set': 93.8},
    {'tdb': 77, 'tr': 77, 'v': 29.5 / 60, 'rh': 10, 'met': 1, 'clo': 0.5, 'set': 74.0},
    {'tdb': 77, 'tr': 77, 'v': 29.5 / 60, 'rh': 90, 'met': 1, 'clo': 0.5, 'set': 76.8},
    {'tdb': 77, 'tr': 77, 'v': 19.7 / 60, 'rh': 50, 'met': 1, 'clo': 0.5, 'set': 75.2},
    {'tdb': 77, 'tr': 77, 'v': 118.1 / 60, 'rh': 50, 'met': 1, 'clo': 0.5, 'set': 70.4},
    {'tdb': 77, 'tr': 77, 'v': 216.5 / 60, 'rh': 50, 'met': 1, 'clo': 0.5, 'set': 68.4},
    {'tdb': 77, 'tr': 77, 'v': 590.6 / 60, 'rh': 50, 'met': 1, 'clo': 0.5, 'set': 65.6},
    {'tdb': 77, 'tr': 50, 'v': 29.5 / 60, 'rh': 50, 'met': 1, 'clo': 0.5, 'set': 59.6},
    {'tdb': 77, 'tr': 104, 'v': 29.5 / 60, 'rh': 50, 'met': 1, 'clo': 0.5, 'set': 88.9},
    {'tdb': 77, 'tr': 77, 'v': 29.5 / 60, 'rh': 50, 'met': 1, 'clo': 0.1, 'set': 69.3},
    {'tdb': 77, 'tr': 77, 'v': 29.5 / 60, 'rh': 50, 'met': 1, 'clo': 1, 'set': 81.0},
    {'tdb': 77, 'tr': 77, 'v': 29.5 / 60, 'rh': 50, 'met': 1, 'clo': 2, 'set': 90.3},
    {'tdb': 77, 'tr': 77, 'v': 29.5 / 60, 'rh': 50, 'met': 1, 'clo': 4, 'set': 99.7},
    {'tdb': 77, 'tr': 77, 'v': 29.5 / 60, 'rh': 50, 'met': 0.8, 'clo': 0.5, 'set': 73.9},
    {'tdb': 77, 'tr': 77, 'v': 29.5 / 60, 'rh': 50, 'met': 2, 'clo': 0.5, 'set': 78.7},
    {'tdb': 77, 'tr': 77, 'v': 29.5 / 60, 'rh': 50, 'met': 4, 'clo': 0.5, 'set': 86.8},
    ]

data_test_pmv_ip = [  # I have commented the lines of code that don't pass the test
    {'tdb': 67.3, 'rh': 86, 'vr': 20 / 60, 'met': 1.1, 'clo': 1, 'pmv': -0.5, 'ppd': 10},
    {'tdb': 75.0, 'rh': 66, 'vr': 20 / 60, 'met': 1.1, 'clo': 1, 'pmv': 0.5, 'ppd': 10},
    {'tdb': 78.2, 'rh': 15, 'vr': 20 / 60, 'met': 1.1, 'clo': 1, 'pmv': 0.5, 'ppd': 10},
    {'tdb': 70.2, 'rh': 20, 'vr': 20 / 60, 'met': 1.1, 'clo': 1, 'pmv': -0.5, 'ppd': 10},
    {'tdb': 74.5, 'rh': 67, 'vr': 20 / 60, 'met': 1.1, 'clo': .5, 'pmv': -0.5, 'ppd': 10},
    {'tdb': 80.2, 'rh': 56, 'vr': 20 / 60, 'met': 1.1, 'clo': .5, 'pmv': 0.5, 'ppd': 10},
    {'tdb': 82.2, 'rh': 13, 'vr': 20 / 60, 'met': 1.1, 'clo': .5, 'pmv': 0.5, 'ppd': 10},
    {'tdb': 76.5, 'rh': 16, 'vr': 20 / 60, 'met': 1.1, 'clo': .5, 'pmv': -0.5, 'ppd': 10},
    ]
# fmt: on


class TestPmvPpd:

    #  Returns a dictionary with 'pmv' and 'ppd' keys
    def test_returns_dictionary_with_pmv_and_ppd_keys(self):
        # Arrange
        tdb = [25]
        tr = [23]
        vr = [0.5]
        rh = [50]
        met = [1.2]
        clo = [0.5]

        # Act
        result = pmv_ppd(tdb, tr, vr, rh, met, clo)

        # Assert
        assert isinstance(result, dict)
        assert "pmv" in result.keys()
        assert "ppd" in result.keys()

    #  Calculates PMV and PPD values for valid input values
    def test_calculates_pmv_and_ppd_values_for_valid_input_values(self):
        # Arrange
        tdb = [25]
        tr = [23]
        vr = [0.5]
        rh = [50]
        met = [1.2]
        clo = [0.5]

        # Act
        result = pmv_ppd(tdb, tr, vr, rh, met, clo)

        # Assert
        assert math.isclose(result["pmv"][0], -0.82)
        assert math.isclose(result["ppd"][0], 19)

    #  Raises a ValueError if standard is not ISO or ASHRAE
    def test_raises_value_error_if_standard_is_not_iso_or_ashrae(self):
        # Arrange
        tdb = [25]
        tr = [23]
        vr = [0.5]
        rh = [50]
        met = [1.2]
        clo = [0.5]

        # Act & Assert
        with pytest.raises(ValueError):
            pmv_ppd(tdb, tr, vr, rh, met, clo, standard="Invalid")

    #  Returns NaN for invalid input values
    def test_returns_nan_for_invalid_input_values(self):
        # Arrange
        tdb = [25, 50]
        tr = [23, 45]
        vr = [0.5, 3]
        rh = [50, 80]
        met = [1.2, 2.5]
        clo = [0.5, 1.8]

        # Act
        result = pmv_ppd(tdb, tr, vr, rh, met, clo)

        # Assert
        assert math.isnan(result["pmv"][1])
        assert math.isnan(result["ppd"][1])

    def test_pmv_ppd(self):
        for table in reference_tables["reference_data"]["pmv_ppd"]:
            for entry in table["data"]:
                standard = "ISO"
                if "ASHRAE" in table["source"]:
                    standard = "ASHRAE"
                inputs = entry["inputs"]
                outputs = entry["outputs"]
                r = pmv_ppd(
                    inputs["ta"],
                    inputs["tr"],
                    inputs["v"],
                    inputs["rh"],
                    inputs["met"],
                    inputs["clo"],
                    standard=standard,
                )
                # asserting with this strange code otherwise face issues with rounding fund
                assert float("%.1f" % r["pmv"]) == outputs["pmv"]
                assert np.round(r["ppd"], 1) == outputs["ppd"]

        for row in data_test_pmv_ip:
            assert (
                abs(
                    round(
                        pmv_ppd(
                            row["tdb"],
                            row["tdb"],
                            row["vr"],
                            row["rh"],
                            row["met"],
                            row["clo"],
                            standard="ashrae",
                            units="ip",
                        )["pmv"],
                        1,
                    )
                    - row["pmv"]
                )
                < 0.011
            )
            assert (
                abs(
                    round(
                        pmv_ppd(
                            row["tdb"],
                            row["tdb"],
                            row["vr"],
                            row["rh"],
                            row["met"],
                            row["clo"],
                            standard="ashrae",
                            units="ip",
                        )["ppd"],
                        1,
                    )
                    - row["ppd"]
                )
                < 1
            )

        assert (
            round(pmv_ppd(67.28, 67.28, 0.328084, 86, 1.1, 1, units="ip")["pmv"], 1)
        ) == -0.5

        np.testing.assert_equal(
            np.around(
                pmv_ppd([70, 70], 67.28, 0.328084, 86, 1.1, 1, units="ip")["pmv"], 1
            ),
            [-0.3, -0.3],
        )

        # test airspeed limits
        np.testing.assert_equal(
            pmv_ppd(
                [26, 24, 22, 26, 24, 22],
                [26, 24, 22, 26, 24, 22],
                [0.9, 0.6, 0.3, 0.9, 0.6, 0.3],
                50,
                [1.1, 1.1, 1.1, 1.3, 1.3, 1.3],
                [0.5, 0.5, 0.5, 0.7, 0.7, 0.7],
                standard="ashrae",
                airspeed_control=False,
            )["pmv"],
            [np.nan, np.nan, np.nan, -0.14, -0.43, -0.57],
        )

        with pytest.raises(ValueError):
            pmv_ppd(25, 25, 0.1, 50, 1.1, 0.5, standard="random")

        # checking that returns np.nan when outside standard applicability limits
        np.testing.assert_equal(
            pmv_ppd(
                [31, 20, 20, 20, 20, 30],
                [20, 41, 20, 20, 20, 20],
                [0.1, 0.1, 2, 0.1, 0.1, 0.1],
                50,
                [1.1, 1.1, 1.1, 0.7, 1.1, 4.1],
                [0.5, 0.5, 0.5, 0.5, 2.1, 0.1],
                standard="iso",
            ),
            {
                "pmv": [np.nan, np.nan, np.nan, np.nan, np.nan, np.nan],
                "ppd": [np.nan, np.nan, np.nan, np.nan, np.nan, np.nan],
            },
        )

        np.testing.assert_equal(
            pmv_ppd(
                [41, 20, 20, 20, 20, 39],
                [20, 41, 20, 20, 20, 39],
                [0.1, 0.1, 2.1, 0.1, 0.1, 0.1],
                50,
                [1.1, 1.1, 1.1, 0.7, 1.1, 3.9],
                [0.5, 0.5, 0.5, 0.5, 2.1, 1.9],
                standard="ashrae",
            ),
            {
                "pmv": [np.nan, np.nan, np.nan, np.nan, np.nan, np.nan],
                "ppd": [np.nan, np.nan, np.nan, np.nan, np.nan, np.nan],
            },
        )

        # check results with limit_inputs disabled
        np.testing.assert_equal(
            pmv_ppd(31, 41, 2, 50, 0.7, 2.1, standard="iso", limit_inputs=False),
            {"pmv": 2.4, "ppd": 91.0},
        )

        np.testing.assert_equal(
            pmv_ppd(41, 41, 2, 50, 0.7, 2.1, standard="ashrae", limit_inputs=False),
            {"pmv": 4.48, "ppd": 100.0},
        )

        for table in reference_tables["reference_data"]["pmv_ppd"]:
            standard = "ISO"
            if "ASHRAE" in table["source"]:
                standard = "ASHRAE"
            tdb = np.array([d["inputs"]["ta"] for d in table["data"]])
            tr = np.array([d["inputs"]["tr"] for d in table["data"]])
            v = np.array([d["inputs"]["v"] for d in table["data"]])
            rh = np.array([d["inputs"]["rh"] for d in table["data"]])
            met = np.array([d["inputs"]["met"] for d in table["data"]])
            clo = np.array([d["inputs"]["clo"] for d in table["data"]])
            pmv_exp = np.array([d["outputs"]["pmv"] for d in table["data"]])
            ppd_exp = np.array([d["outputs"]["ppd"] for d in table["data"]])
            results = pmv_ppd(tdb, tr, v, rh, met, clo, standard=standard)
            pmv_r = [float("%.1f" % x) for x in results["pmv"]]

            np.testing.assert_equal(pmv_r, pmv_exp)
            np.testing.assert_equal(results["ppd"], ppd_exp)

    def test_pmv_ppd_optimized(self):
        assert math.isclose(
            _pmv_ppd_optimized(25, 25, 0.3, 50, 1.5, 0.7, 0), 0.55, abs_tol=0.01
        )

        np.testing.assert_equal(
            np.around(_pmv_ppd_optimized([25, 25], 25, 0.3, 50, 1.5, 0.7, 0), 2),
            [0.55, 0.55],
        )


class TestPmvPpdOptimized:

    #  The function returns the correct PMV value for typical input values.
    def test_pmv_typical_input(self):
        # Typical input values
        tdb = 25
        tr = 23
        vr = 0.1
        rh = 50
        met = 1.2
        clo = 0.5
        wme = 0

        expected_pmv = -0.197

        assert math.isclose(
            _pmv_ppd_optimized(tdb, tr, vr, rh, met, clo, wme),
            expected_pmv,
            abs_tol=0.01,
        )

    #  The function returns the correct PMV value for extreme input values.
    def test_pmv_extreme_input(self):
        # Extreme input values
        tdb = 35
        tr = 45
        vr = 2
        rh = 10
        met = 2.5
        clo = 1.5
        wme = 1

        expected_pmv = 1.86

        assert math.isclose(
            _pmv_ppd_optimized(tdb, tr, vr, rh, met, clo, wme),
            expected_pmv,
            abs_tol=0.01,
        )

    #  The function returns NaN if any of the input values are NaN.
    def test_nan_input_values(self):
        # Input values with NaN
        tdb = float("nan")
        tr = 23
        vr = 0.1
        rh = 50
        met = 1.2
        clo = 0.5
        wme = 0

        assert math.isnan(_pmv_ppd_optimized(tdb, tr, vr, rh, met, clo, wme))

    #  The function returns NaN if any of the input values are infinite.
    def test_infinite_input_values(self):
        # Input values with infinity
        tdb = float("inf")
        tr = 23
        vr = 0.1
        rh = 50
        met = 1.2
        clo = 0.5
        wme = 0

        assert math.isnan(_pmv_ppd_optimized(tdb, tr, vr, rh, met, clo, wme))

    #  Returns a dictionary with 'pmv' and 'ppd' keys
    def test_check_wrong_input(self):
        with pytest.raises(TypeError):
            pmv_ppd(25, 25, 0.1, 50, 1.1, 0.5, stardard="random")



import logging
import numpy as np
from pythermalcomfort.models.jos3 import JOS3
from pythermalcomfort.jos3_functions.parameters import Default
from pythermalcomfort.models.zhang_comfort import zhang_sensation_comfort
from pythermalcomfort.utilities import DefaultSkinTemperature

# Set logging config as WARNING
logging.basicConfig(level=logging.WARNING)


class ATCS(JOS3):
    """ATCS (Advanced Thermal Comfort Simulator) is an integrated simulator that combines
    JOS-3 model for simulating human thermal physiology, with a thermal sensation and
    comfort model for predicting psychological parameters from the physiological parameters.
    """

    default_iteration_number_to_get_stable_condition = 600

    def __init__(self, *args, **kwargs):
        """Initializes this class by calling the parent class constructor
        and setting initial values for instance variables.

        Parameters
        ----------
        height : float, optional
            body height, in [m]. The default is 1.72.
        weight : float, optional
            body weight, in [kg]. The default is 74.43.
        fat : float, optional
            fat percentage, in [%]. The default is 15.
        age : int, optional
            age, in [years]. The default is 20.
        sex : str, optional
            sex ("male" or "female"). The default is "male".
        ci : float, optional
            Cardiac index, in [L/min/m2]. The default is 2.6432.
        bmr_equation : str, optional
            The equation used to calculate basal metabolic rate (BMR). Choose a BMR equation.
            The default is "harris-benedict" equation created uding Caucasian's data. (DOI: doi.org/10.1073/pnas.4.12.370)
            If the Ganpule's equation (DOI: doi.org/10.1038/sj.ejcn.1602645) for Japanese people is used, input "japanese".
        bsa_equation : str, optional
            The equation used to calculate body surface area (bsa). Choose a bsa equation.
            You can choose "dubois", "fujimoto", "kruazumi", or "takahira". The default is "dubois".
            The body surface area can be calculated using the function
            :py:meth:`pythermalcomfort.utilities.body_surface_area`.
        ex_output : None, list or "all", optional
            This is used when you want to display results other than the default output parameters (ex.skin temperature);
            by default, JOS outputs only the most necessary parameters in order to reduce the computational load.
            If the parameters other than the default output parameters are needed,
            specify the list of the desired parameter names in string format like ["bf_skin", "bf_core", "t_artery"].
            If you want to display all output results, set ex_output is "all".

        Attributes
        ----------
        tdb : float, int, or array-like
            Dry bulb air temperature [°C].
        tr : float, int, or array-like
            Mean radiant temperature [°C].
        to : float, int, or array-like
            Operative temperature [°C].
        v : float, int, or array-like
            Air speed [m/s].
        rh : float, int, or array-like
            Relative humidity [%].
        clo : float, int, or array-like
            Clothing insulation [clo].
            Note: If you want to input clothing insulation to each body part,
            it can be input using the dictionaly in "utilities.py" in "jos3_function" folder.
            :py:meth:`pythermalcomfort.jos3_functions.utilities.local_clo_typical_ensembles`.
        par : float
            Physical activity ratio [-].
            This equals the ratio of metabolic rate to basal metabolic rate.
            The par of sitting quietly is 1.2.
        posture : str
            Body posture [-]. Choose a posture from standing, sitting or lying.
        body_temp : numpy.ndarray (85,)
            All segment temperatures of JOS-3

        Methods
        -------
        simulate(times, dtime, output):
            Run this model for given times.
        dict_results():
            Get results as a dictionary with pandas.DataFrame values.

        Returns
        -------
        Predicted thermal phygiology
            The same output as JOS-3 model
        Predicted thermal sensation and comfort
            Outputs from Zhang's sensation and comfort model with the inputs of JOS-3 model

        Examples
        -------
        .. code-block:: python
        >>> from pythermalcomfort.models import atcs
        >>> import pandas as pd
        >>> import matplotlib.pyplot as plt
        >>> import os
        >>>
        >>># Build a model and set a body built
        >>># Create an instance of the class with optional body parameters such as body height, weight, age, sex, etc.
        >>> model = atcs.ATCS(height=1.7, weight=60, age=30)
        >>>
        >>># Set the first phase
        >>> model.to = 28  # Operative temperature [°C]
        >>> model.rh = 40  # Relative humidity [%]
        >>> model.v = 0.2  # Air velocity [m/s]
        >>> model.par = 1.2  # Physical activity ratio [-]
        >>> model.simulate(60)  # Exposure time = 60 [min]
        >>>
        >>># Set the next condition (You only need to change the parameters that you want to change)
        >>> model.to = 20  # Change only operative temperature
        >>> model.simulate(60)  # Additional exposure time = 60 [min]
        >>>
        >>># Show the results
        >>> df = pd.DataFrame(model.dict_results())
        >>> df["sensation_overall"].plot()
        >>> plt.ylabel("Overall thermal sensation [-]")
        >>> plt.xlabel("Time [min]")
        >>> plt.savefig("atcs_example_local_sensations.png"))
        >>> plt.show()
        >>>
        >>># Exporting the results as csv
        >>> model.to_csv("atcs_example1 (default output).csv"))
        """
        # Initialize instance variables for this class.
        self.cached_clo = np.ones(17) * Default.clothing_insulation
        self.cached_par = Default.physical_activity_ratio
        self.cached_overall_clo = np.average(self.cached_clo, weights=Default.local_bsa)

        # Initialize parent class
        self.comfort_setpt_skin = np.array(list(DefaultSkinTemperature()._asdict()))
        self.t_skin_used = np.ones(17) * Default.skin_temperature
        self.t_skin_wet = np.ones(17) * Default.skin_temperature
        self.t_skin_previous_time_step = None
        self.t_core_previous_time_step = None
        self.overall_clo = Default.clothing_insulation

        # Initialize parent class
        super().__init__(*args, **kwargs)

        self.overall_clo = np.average(self.clo, weights=Default.local_bsa)
        logging.info(f"During initialization, overall clo value is {self.overall_clo}")

    def _calculate_comfort_setpt(self, overall_clo, par):
        """Computes and returns the skin temperature when thermally neutral.

        This method runs the JOS-3 model in uniform conditions at a temperature where PMV=0
        for the provided metabolic rate and clothing insulation values. If the given values match
        the cached ones, it directly returns the cached comfortable skin temperature set point.
        Otherwise, it calculates and returns a new comfortable skin temperature set point.

        Parameters
        ----------
        overall_clo : float
            Overall clothing insulation value [clo]
        par : float
            Physical activity ratio [-]

        Returns
        -------
        comfort_setpt_skin : np.array
            Skin temperature set point for sensation/comfort model

        Notes
        -----
        - If both `clo_overall` and `par` values match the cached values, this function will return
          the cached comfortable skin temperature set point directly.
        - The calculation of the comfortable skin temperature set point involves determining the operative temperature
          where PMV is zero and then running the model in uniform conditions to extract skin temperatures as set points.
        """
        # If the clothing insulation value 'clo' and the metabolic rate 'par' are the same as the cached values,
        # return the cached comfortable skin temperature set point
        if overall_clo == self.cached_overall_clo and par == self.cached_par:
            # print("Both 'clo' and 'par' are the same as the cached values. Cached values are applied to comfort set-points")
            return self.comfort_setpt_skin

        # Otherwise, calculate a new comfortable skin temperature set point
        else:
            # Calculate the metabolic rate based on the provided 'par' value.
            min_met = 1.0
            met = max(self.bmr * self.par / 58.15, min_met)

            # Calculate the metabolic rate based on the provided 'par' value.
            overall_clo = np.average(self._clo, weights=Default.local_bsa)

            # Cash PAR (physical activity ratio) and overall clothing insulation
            self.cached_par = self.par
            self.cached_overall_clo = self.overall_clo

            # Use the parent class's method to calculate the operative temperature where PMV is zero.
            self.to = self._calculate_operative_temp_when_pmv_is_zero(
                met=met, clo=overall_clo
            )

            # Run the model in uniform conditions at a temperature where PMV=0
            # for the given met and clo and use the resulting skin temperatures as the setpoints
            # Assuming that the initial conditions are uniform and at a temperature where PMV=0
            logging.debug("Skin neutral temperature calculation starts")
            logging.debug("Metabolic rate:", met)
            logging.debug("Overall clo value:", overall_clo)
            logging.debug("Operative temperature when PMV=0", self.to)

            for _ in range(10):
                self._run(dtime=600, passive=False)

            self.comfort_setpt_skin = self.t_skin
            logging.debug("Comfort set-point temperature:", self.comfort_setpt_skin)

            return self.comfort_setpt_skin

    def _calculate_skin_temperature_considering_sweating(
        self,
        t_skin,
        wet,
    ):
        """Calculate the skin temperature considering sweating for Zhang's model.

        This function takes into account the impact of sweating on the skin temperature. The Zhang model's
        prediction of local thermal sensation is represented by a logistic function of the difference between
        the current skin temperature and the neutral skin temperature. In a hot environment, sweating tends to
        suppress the increase in skin temperature, resulting in smaller differences in skin temperatures at
        locations with higher skin temperatures (like the chest or back) under neutral thermal conditions. This
        can lead to an underestimation of thermal sensations. Hence, this function modifies the skin temperature
        based on skin wettedness, utilizing a simple consideration.
        Based on the relationship between skin temperature and wettedness at isothermal sensation,
        skin temperature is corrected by increasing skin wettedness to improve prediction accuracy at warm environments.

        More details can be found in this Japanese research paper and data for TSV=3 is used as a reference:
        https://eprints.lib.hokudai.ac.jp/dspace/bitstream/2115/7826/1/4-2-6_p67-72.pdf

        Parameters
        ----------
        t_skin : float or numpy.ndarray
            Skin temperature [°C]
        wet : float
            skin wettedness [-]

        Returns
        -------
        t_skin_wet : float or numpy.ndarray
            Corrected skin temperature, considering latent heat [°C]

        Notes
        -----
        In the future, it may be more reasonable to predict thermal sensation
        based on the total heat loss (sensible + latent) of JOS3 model, as in the SET calculation method.
        """
        coef = 2.5
        t_skin_increase = coef * (wet - 0.06)
        t_skin_wet = t_skin + t_skin_increase
        return t_skin_wet

    def _run(self, *args, **kwargs):
        """Run the simulation for a single time step.

        This method extends the `_run` method of the parent class. In addition to the parent's operations, it
        calculates the rate of change of skin and core temperature and estimates thermal sensation and comfort parameters.

        Parameters
        ----------
        *args :
            Variable length argument list.
        **kwargs :
            Arbitrary keyword arguments.

        Returns
        -------
        dict_data : dict
            Dictionary containing data from the parent class `_run` method, rate of change of skin and core temperature,
            and other thermal sensation and comfort parameters.

        Notes
        -----
        - The skin temperature after considering sweat (`t_skin_wet`) is used for further calculations if `consider_sweat`
          is True. Otherwise, the regular skin temperature (`t_skin`) is used.
        - Local and overall thermal sensations and comfort are computed using the `cbe_comfort` function.
        """

        # Internal switch to consider sweat for comfort model
        consider_sweat = True

        # Define comfort set-point temperature
        comfort_setpt_skin = self._calculate_comfort_setpt(
            overall_clo=self.overall_clo, par=self.par
        )
        # Calculate the skin temperature after accounting for the effects of sweating.
        self.t_skin_wet = self._calculate_skin_temperature_considering_sweating(
            t_skin=self.t_skin, wet=self.w
        )

        # Use t_skin_wet for calculations if consider_sweat is True
        self.t_skin_used = self.t_skin_wet if consider_sweat else self.t_skin

        # Execute the `_run` method from the parent class and store its returned data.
        dict_data = super()._run(*args, **kwargs)

        # Retrieve or default the time difference value from keyword arguments.
        dtime = kwargs.get("dtime", 60)

        # Calculate the rate of change of skin and core temperatures over time.
        dt_skin = (
            (self.t_skin_used - self.t_skin_previous_time_step) / dtime
            if self.t_skin_previous_time_step is not None
            else 0
        )
        dt_core = (
            (self.t_core - self.t_core_previous_time_step) / dtime
            if self.t_core_previous_time_step is not None
            else 0
        )

        # Store the used skin and core temperatures for future calculations.
        self.t_skin_previous_time_step = self.t_skin_used
        self.t_core_previous_time_step = self.t_core

        # Update the dictionary with the rate of change values for skin and core temperatures.
        dict_data.update(
            {
                "dt_skin": dt_skin,
                "dt_core": dt_core,
            }
        )

        # Calculate thermal sensations and comfort parameters
        dict_sensation_and_comfort = zhang_sensation_comfort(
            t_skin_local=self.t_skin_used,
            dt_skin_local_dt=dt_skin,
            t_skin_local_set=comfort_setpt_skin,
            dt_core_local_dt=dt_core,
        )

        # Update the dictionary with the thermal sensation and comfort parameters
        dict_data.update(dict_sensation_and_comfort)

        return dict_data

    def simulate(self, *args, **kwargs):
        """Runs the model for a given time.
        This method extends the simulate method of the parent class by additionally computing the comfortable skin
        temperature set point before running the simulation.
        """
        result = super().simulate(*args, **kwargs)
        return result



from pythermalcomfort.psychrometrics import psy_ta_rh


def at(tdb, rh, v, q=None, **kwargs):
    """Calculates the Apparent Temperature (AT). The AT is defined as the
    temperature at the reference humidity level producing the same amount of
    discomfort as that experienced under the current ambient temperature,
    humidity, and solar radiation [17]_. In other words, the AT is an
    adjustment to the dry bulb temperature based on the relative humidity
    value. Absolute humidity with a dew point of 14°C is chosen as a reference.

    [16]_. It includes the chilling effect of the wind at lower temperatures.

    Two formulas for AT are in use by the Australian Bureau of Meteorology: one includes
    solar radiation and the other one does not (http://www.bom.gov.au/info/thermal_stress/
    , 29 Sep 2021). Please specify q if you want to estimate AT with solar load.

    Parameters
    ----------
    tdb : float
        dry bulb air temperature,[°C]
    rh : float
        relative humidity, [%]
    v : float
        wind speed 10m above ground level, [m/s]
    q : float
        Net radiation absorbed per unit area of body surface [W/m2]

    Other Parameters
    ----------------
    round: boolean, default True
        if True rounds output value, if False it does not round it

    Returns
    -------
    at: float
        apparent temperature, [°C]

    Examples
    --------
    .. code-block:: python

        >>> from pythermalcomfort.models import at
        >>> at(tdb=25, rh=30, v=0.1)
        24.1
    """
    default_kwargs = {
        "round": True,
    }
    kwargs = {**default_kwargs, **kwargs}

    # dividing it by 100 since the at eq. requires p_vap to be in hPa
    p_vap = psy_ta_rh(tdb, rh).p_vap / 100

    # equation sources [16] and http://www.bom.gov.au/info/thermal_stress/#apparent
    if q:
        t_at = tdb + 0.348 * p_vap - 0.7 * v + 0.7 * q / (v + 10) - 4.25
    else:
        t_at = tdb + 0.33 * p_vap - 0.7 * v - 4.00

    if kwargs["round"]:
        t_at = round(t_at, 1)

    return t_at



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function
from __future__ import annotations

import io
import re
from glob import glob
from os.path import basename
from os.path import dirname
from os.path import join
from os.path import splitext

from setuptools import find_packages
from setuptools import setup


def read(*names, **kwargs):
    with io.open(
        join(dirname(__file__), *names), encoding=kwargs.get("encoding", "utf8")
    ) as fh:
        return fh.read()


setup(
    name="pythermalcomfort",
    version="2.10.0",
    license="MIT",
    description=(
        "Package to calculate several thermal comfort indices (e.g. PMV, PPD, SET,"
        " adaptive) and convert physical variables. Please cite us if you use this"
        " package: Tartarini, F., Schiavon, S., 2020. pythermalcomfort: A Python"
        " package for thermal comfort research. SoftwareX 12, 100578."
        " https://doi.org/10.1016/j.softx.2020.100578"
    ),
    long_description="%s\n%s"
    % (
        re.compile("^.. start-badges.*^.. end-badges", re.M | re.S).sub(
            "", read("README.rst")
        ),
        re.sub(":[a-z]+:`~?(.*?)`", r"``\1``", read("CHANGELOG.rst")),
    ),
    author="Federico Tartarini",
    author_email="cbecomforttool@gmail.com",
    url="https://github.com/CenterForTheBuiltEnvironment/pythermalcomfort",
    # packages=find_packages("pythermalcomfort"),
    # package_dir={"pythermalcomfort": "pythermalcomfort"},
    # py_modules=[splitext(basename(path))[0] for path in glob("pythermalcomfort/*.py")],
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        # complete classifier list: http://pypi.python.org/pypi?%3Aaction=list_classifiers
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Intended Audience :: Education",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Unix",
        "Operating System :: POSIX",
        "Operating System :: Microsoft :: Windows",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Scientific/Engineering",
        "Topic :: Utilities",
    ],
    project_urls={
        "Documentation": "https://pythermalcomfort.readthedocs.io/",
        "Changelog": "https://pythermalcomfort.readthedocs.io/en/latest/changelog.html",
        "Issue Tracker": (
            "https://github.com/CenterForTheBuiltEnvironment/pythermalcomfort/issues"
        ),
    },
    keywords=[
        "thermal comfort",
        "pmv",
        "ppd",
        "building design",
        "compliance",
        "thermal environment",
        "built environment",
    ],
    python_requires=">=3.8.0",
    install_requires=[
        "scipy",
        "numba",
        "numpy",
        "setuptools",
    ],  # eg: 'aspectlib==1.1.1', 'six>=1.7',
    extras_require={
        # eg:
        #   'rst': ['docutils>=0.11'],
        #   ':python_version=="2.6"': ['argparse'],
    },
    entry_points={
        "console_scripts": [
            "pythermalcomfort = pythermalcomfort.cli:main",
        ]
    },
)
